@@ -8,9 +8,10 @@
 """KiBot: KiCad automation tool for documents generation
 
 Usage:
-  kibot [-b BOARD] [-e SCHEMA] [-c CONFIG] [-d OUT_DIR] [-s PRE]
-         [-q | -v...] [-C | -i | -n] [-m MKFILE] [-g DEF]... [TARGET...]
-  kibot [-v...] [-b BOARD] [-e SCHEMA] [-c PLOT_CONFIG] --list
+  kibot [-b BOARD] [-e SCHEMA] [-c CONFIG] [-d OUT_DIR] [-s PRE] [-D]
+         [-q | -v...] [-C | -i | -n] [-m MKFILE] [-A] [-g DEF] ...
+         [-E DEF] ... [-w LIST] [TARGET...]
+  kibot [-v...] [-b BOARD] [-e SCHEMA] [-c PLOT_CONFIG] [-E DEF] ... --list
   kibot [-v...] [-b BOARD] [-d OUT_DIR] [-p | -P] --example
   kibot [-v...] [--start PATH] [-d OUT_DIR] [--dry] [-t, --type TYPE]...
          --quick-start
@@ -21,6 +22,7 @@ Usage:
   kibot [-v...] --help-output=HELP_OUTPUT
   kibot [-v...] --help-outputs
   kibot [-v...] --help-preflights
+  kibot [-v...] --help-variants
   kibot -h | --help
   kibot --version
 
@@ -28,11 +30,14 @@ Arguments:
   TARGET    Outputs to generate, default is all
 
 Options:
+  -A, --no-auto-download           Disable dependencies auto-download
   -b BOARD, --board-file BOARD     The PCB .kicad-pcb board file
   -c CONFIG, --plot-config CONFIG  The plotting config file to use
   -C, --cli-order                  Generate outputs using the indicated order
   -d OUT_DIR, --out-dir OUT_DIR    The output directory [default: .]
+  -D, --dont-stop                  Try to continue if an output fails
   -e SCHEMA, --schematic SCHEMA    The schematic file (.sch)
+  -E DEF, --define DEF             Define preprocessor value (VAR=VAL)
   -g DEF, --global-redef DEF       Overwrite a global value (VAR=VAL)
   -i, --invert-sel                 Generate the outputs not listed as targets
   -l, --list                       List available outputs (in the config file)
@@ -44,6 +49,7 @@ Options:
   -s PRE, --skip-pre PRE           Skip preflights, comma separated or `all`
   -v, --verbose                    Show debugging information
   -V, --version                    Show program's version number and exit
+  -w, --no-warn LIST               Exclude the mentioned warnings (comma sep)
   -x, --example                    Create a template configuration file
 
 Quick start options:
@@ -61,20 +67,22 @@ Help options:
   --help-output HELP_OUTPUT        Help for this particular output
   --help-outputs                   List supported outputs and details
   --help-preflights                List supported preflights and details
+  --help-variants                  List supported variants and details
 
 """
-import os
-import sys
-from sys import path as sys_path
-import re
+from glob import glob
 import gzip
 import locale
-from glob import glob
+import os
+import re
+import sys
+from sys import path as sys_path
 from . import __version__, __copyright__, __license__
 # Import log first to set the domain
 from . import log
 log.set_domain('kibot')
 logger = log.init()
+from . import dep_downloader
 from .docopt import docopt
 # GS will import pcbnew, so we must solve the nightly setup first
 # Check if we have to run the nightly KiCad build
@@ -92,8 +100,9 @@ if os.environ.get('KIAUS_USE_NIGHTLY'):  # pragma: no cover (nightly)
 from .gs import GS
 from .misc import EXIT_BAD_ARGS, W_VARCFG, NO_PCBNEW_MODULE, W_NOKIVER, hide_stderr, TRY_INSTALL_CHECK
 from .pre_base import BasePreFlight
+from .error import KiPlotConfigurationError, config_error
 from .config_reader import (CfgYamlReader, print_outputs_help, print_output_help, print_preflights_help, create_example,
-                            print_filters_help, print_global_options_help, print_dependencies)
+                            print_filters_help, print_global_options_help, print_dependencies, print_variants_help)
 from .kiplot import (generate_outputs, load_actions, config_output, generate_makefile, generate_examples, solve_schematic,
                      solve_board_file, solve_project_file, check_board_file)
 GS.kibot_version = __version__
@@ -181,13 +190,15 @@ def detect_kicad():
     GS.kicad_version_minor = int(m.group(2))
     GS.kicad_version_patch = int(m.group(3))
     GS.kicad_version_n = GS.kicad_version_major*1000000+GS.kicad_version_minor*1000+GS.kicad_version_patch
+    GS.ki6 = GS.kicad_version_major >= 6
+    GS.ki5 = GS.kicad_version_major < 6
     logger.debug('Detected KiCad v{}.{}.{} ({} {})'.format(GS.kicad_version_major, GS.kicad_version_minor,
                  GS.kicad_version_patch, GS.kicad_version, GS.kicad_version_n))
     # Used to look for plug-ins.
     # KICAD_PATH isn't good on my system.
     # The kicad-nightly package overwrites the regular package!!
     GS.kicad_share_path = '/usr/share/kicad'
-    if GS.ki6():
+    if GS.ki6:
         GS.kicad_conf_path = pcbnew.GetSettingsManager().GetUserSettingsPath()
         if nightly:
             # Nightly Debian packages uses `/usr/share/kicad-nightly/kicad-nightly.env` as an environment extension
@@ -225,13 +236,49 @@ def detect_kicad():
         GS.kicad_plugins_dirs.append(os.path.join(home, '.kicad', 'scripting'))
         GS.kicad_plugins_dirs.append(os.path.join(home, '.kicad', 'scripting', 'plugins'))
         if GS.kicad_version_major >= 6:
-            # KiCad 6.0 PCM
             ver_dir = str(GS.kicad_version_major)+'.'+str(GS.kicad_version_minor)
-            GS.kicad_plugins_dirs.append(os.path.join(home, '.local', 'share', 'kicad', ver_dir, '3rdparty', 'plugins'))
+            local_share = os.path.join(home, '.local', 'share', 'kicad', ver_dir)
+            GS.kicad_plugins_dirs.append(os.path.join(local_share, 'scripting'))
+            GS.kicad_plugins_dirs.append(os.path.join(local_share, 'scripting', 'plugins'))
+            GS.kicad_plugins_dirs.append(os.path.join(local_share, '3rdparty', 'plugins'))   # KiCad 6.0 PCM
     if GS.debug_level > 1:
         logger.debug('KiCad config path {}'.format(GS.kicad_conf_path))
 
 
+def parse_defines(args):
+    for define in args.define:
+        if '=' not in define:
+            logger.error('Malformed `define` option, must be VARIABLE=VALUE ({})'.format(define))
+            sys.exit(EXIT_BAD_ARGS)
+        var = define.split('=')[0]
+        GS.cli_defines[var] = define[len(var)+1:]
+
+
+def parse_global_redef(args):
+    for redef in args.global_redef:
+        if '=' not in redef:
+            logger.error('Malformed global-redef option, must be VARIABLE=VALUE ({})'.format(redef))
+            sys.exit(EXIT_BAD_ARGS)
+        var = redef.split('=')[0]
+        GS.cli_global_defs[var] = redef[len(var)+1:]
+
+
+class SimpleFilter(object):
+    def __init__(self, num):
+        self.number = num
+        self.regex = re.compile('')
+        self.error = ''
+
+
+def apply_warning_filter(args):
+    if args.no_warn:
+        try:
+            log.set_filters([SimpleFilter(int(n)) for n in args.no_warn.split(',')])
+        except ValueError:
+            logger.error('-w/--no-warn must specify a comma separated list of numbers ({})'.format(args.no_warn))
+            sys.exit(EXIT_BAD_ARGS)
+
+
 def main():
     set_locale()
     ver = 'KiBot '+__version__+' - '+__copyright__+' - License: '+__license__
@@ -240,17 +287,21 @@ def main():
 
     # Set the specified verbosity
     GS.debug_enabled = log.set_verbosity(logger, args.verbose, args.quiet)
-    GS.debug_level = args.verbose
+    log.debug_level = GS.debug_level = args.verbose
+    logger.debug('KiBot {} verbose level: {}'.format(__version__, args.verbose))
+    apply_warning_filter(args)
     # Now we have the debug level set we can check (and optionally inform) KiCad info
     detect_kicad()
 
+    # Force iBoM to avoid the use of graphical stuff
+    os.environ['INTERACTIVE_HTML_BOM_NO_DISPLAY'] = 'True'
+
     # Parse global overwrite options
-    for redef in args.global_redef:
-        if '=' not in redef:
-            logger.error('Malformed global-redef option, must be VARIABLE=VALUE ({})'.format(redef))
-            sys.exit(EXIT_BAD_ARGS)
-        var = redef.split('=')[0]
-        GS.cli_global_defs[var] = redef[len(var)+1:]
+    parse_global_redef(args)
+
+    # Disable auto-download if needed
+    if args.no_auto_download:
+        dep_downloader.disable_auto_download = True
 
     # Output dir: relative to CWD (absolute path overrides)
     GS.out_dir = os.path.join(os.getcwd(), args.out_dir)
@@ -267,6 +318,9 @@ def main():
     if args.help_preflights:
         print_preflights_help()
         sys.exit(0)
+    if args.help_variants:
+        print_variants_help()
+        sys.exit(0)
     if args.help_filters:
         print_filters_help()
         sys.exit(0)
@@ -297,20 +351,29 @@ def main():
     # Determine the project file
     GS.set_pro(solve_project_file())
 
+    # Parse preprocessor defines
+    parse_defines(args)
+
     # Read the config file
     cr = CfgYamlReader()
     outputs = None
     try:
         # The Python way ...
         with gzip.open(plot_config) as cf_file:
-            outputs = cr.read(cf_file)
+            try:
+                outputs = cr.read(cf_file)
+            except KiPlotConfigurationError as e:
+                config_error(str(e))
     except OSError:
         pass
     if outputs is None:
         with open(plot_config) as cf_file:
-            outputs = cr.read(cf_file)
+            try:
+                outputs = cr.read(cf_file)
+            except KiPlotConfigurationError as e:
+                config_error(str(e))
 
-    # Is just list the available targets?
+    # Is just "list the available targets"?
     if args.list:
         list_pre_and_outs(logger, outputs)
         sys.exit(0)
@@ -320,7 +383,8 @@ def main():
         generate_makefile(args.makefile, plot_config, outputs)
     else:
         # Do all the job (preflight + outputs)
-        generate_outputs(outputs, args.target, args.invert_sel, args.skip_pre, args.cli_order, args.no_priority)
+        generate_outputs(outputs, args.target, args.invert_sel, args.skip_pre, args.cli_order, args.no_priority,
+                         dont_stop=args.dont_stop)
     # Print total warnings
     logger.log_totals()
 
