@@ -13,15 +13,19 @@ import pprint
 import os.path as op
 import sys
 import logging
+import html
 from textwrap import wrap
 from base64 import b64decode
 from .columnlist import ColumnList
 from .kibot_logo import KIBOT_LOGO
 from .. import log
-from ..misc import W_NOKICOST, W_UNKDIST, KICOST_ERROR, W_BADFIELD, TRY_INSTALL_CHECK
+from ..misc import W_NOKICOST, W_UNKDIST, KICOST_ERROR, W_BADFIELD
 from ..error import trace_dump
 from ..gs import GS
 from .. import __version__
+# Init the logger first
+logger = log.get_logger()
+# XLSX Writer support
 try:
     from xlsxwriter import Workbook
     XLSX_SUPPORT = True
@@ -30,8 +34,6 @@ except ModuleNotFoundError:
 
     class Workbook():
         pass
-# Init the logger first
-logger = log.get_logger()
 # KiCost support
 try:
     # Give priority to submodules
@@ -45,13 +47,7 @@ try:
                         init_all_loggers, create_worksheet, Spreadsheet, get_distributors_list, get_dist_name_from_label,
                         set_distributors_progress, is_valid_api)
     KICOST_SUPPORT = True
-except ModuleNotFoundError:
-    KICOST_SUPPORT = False
-    ProgressConsole = object
 except ImportError:
-    logger.error("Installed KiCost is older than the version we support.")
-    logger.error("Try installing the last release or the current GIT code.")
-    logger.error(TRY_INSTALL_CHECK)
     KICOST_SUPPORT = False
     ProgressConsole = object
 
@@ -314,6 +310,8 @@ def create_meta(workbook, name, columns, parts, fmt_head, fmt_cols, max_w, renam
             if not text_l:
                 continue
             fmt_kind = 0 if col_l in SPECS_GENERATED else 2
+            text = html.unescape(text)
+            logger.debugl(3, 'Spec ({},{}): `{}`'.format(r, c, text))
             worksheet.write_string(r+1, c, text, fmt_cols[fmt_kind][r % 2])
             if text_l > col_w[c]:
                 if text_l > max_w:
@@ -709,8 +707,7 @@ def write_xlsx(filename, groups, col_fields, head_names, cfg):
     cfg = BoMOptions object with all the configuration
     """
     if not XLSX_SUPPORT:
-        logger.error('Python xlsxwriter module not installed (Debian: python3-xlsxwriter)')
-        logger.error(TRY_INSTALL_CHECK)
+        logger.error('Python xlsxwriter module not installed')
         return False
 
     link_datasheet = -1
