@@ -9,14 +9,16 @@
 Class to read KiBot config files
 """
 
+import collections
+import io
 import os
 import json
 from sys import (exit, maxsize)
 from collections import OrderedDict
 
-from .error import (KiPlotConfigurationError, config_error)
+from .error import KiPlotConfigurationError
 from .misc import (NO_YAML_MODULE, EXIT_BAD_ARGS, EXAMPLE_CFG, WONT_OVERWRITE, W_NOOUTPUTS, W_UNKOUT, W_NOFILTERS,
-                   W_NOVARIANTS, W_NOGLOBALS, TRY_INSTALL_CHECK)
+                   W_NOVARIANTS, W_NOGLOBALS, TRY_INSTALL_CHECK, W_NOPREFLIGHTS)
 from .gs import GS
 from .registrable import RegOutput, RegVariant, RegFilter, RegDependency
 from .pre_base import BasePreFlight
@@ -29,6 +31,14 @@ LOCAL_OPTIONAL = 1
 GLOBAL_OPTIONAL = LOCAL_OPTIONAL*100
 LOCAL_MANDATORY = GLOBAL_OPTIONAL*100
 GLOBAL_MANDATORY = LOCAL_MANDATORY*100
+GITHUB_RAW = 'https://raw.githubusercontent.com/INTI-CMNB/KiBot/master/docs/images/'
+DEB_LOGO = '![Debian]('+GITHUB_RAW+'debian-openlogo-22x22.png)'
+PYPI_LOGO = ('![PyPi dependency]('+GITHUB_RAW+'PyPI_logo_simplified-22x22.png)')
+PY_LOGO = ('![Python module]('+GITHUB_RAW+'Python-logo-notext-22x22.png)')
+TOOL_LOGO = '![Tool]('+GITHUB_RAW+'llave-inglesa-22x22.png)'
+AUTO_DOWN = '![Auto-download]('+GITHUB_RAW+'auto_download-22x22.png)'
+VALID_SECTIONS = {'kiplot', 'kibot', 'import', 'global', 'filters', 'variants', 'preflight', 'outputs'}
+VALID_KIBOT_SEC = {'version', 'imported_global_has_less_priority'}
 
 
 try:
@@ -40,37 +50,68 @@ except ImportError:
     exit(NO_YAML_MODULE)
 
 
+def update_dict(d, u):
+    for k, v in u.items():
+        if isinstance(v, collections.abc.Mapping):
+            d[k] = update_dict(d.get(k, {}), v)
+        elif isinstance(v, list) and k in d:
+            d[k] = v+d[k]
+        else:
+            d[k] = v
+    return d
+
+
+class CollectedImports(object):
+    def __init__(self):
+        super().__init__()
+        self.outputs = []
+        self.filters = {}
+        self.variants = {}
+        self.globals = {}
+        self.preflights = []
+        self.imported_global_has_less_priority = False
+
+
 class CfgYamlReader(object):
     def __init__(self):
         super().__init__()
         self.imported_globals = {}
         self.no_run_by_default = []
+        self.imported_global_has_less_priority = False
 
     def _check_version(self, v):
         if not isinstance(v, dict):
-            config_error("Incorrect `kibot` section")
+            raise KiPlotConfigurationError("Incorrect `kibot` section")
         if 'version' not in v:
-            config_error("YAML config needs `kibot.version`.")
+            raise KiPlotConfigurationError("YAML config needs `kibot.version`.")
         version = v['version']
         # Only version 1 is known
         if version != 1:
-            config_error("Unknown KiBot config version: "+str(version))
+            raise KiPlotConfigurationError("Unknown KiBot config version: "+str(version))
         return version
 
+    def _check_globals_priority(self, v):
+        ops = 'imported_global_has_less_priority'
+        if ops in v:
+            value = v[ops]
+            if not isinstance(value, bool):
+                raise KiPlotConfigurationError(ops+" must be boolean")
+            self.imported_global_has_less_priority = value
+
     def _parse_output(self, o_tree):
         try:
             name = str(o_tree['name'])
             if not name:
                 raise KeyError
         except KeyError:
-            config_error("Output needs a name in: "+str(o_tree))
+            raise KiPlotConfigurationError("Output needs a name in: "+str(o_tree))
 
         try:
             otype = o_tree['type']
             if not otype:
                 raise KeyError
         except KeyError:
-            config_error("Output `"+name+"` needs a type")
+            raise KiPlotConfigurationError("Output `"+name+"` needs a type")
 
         try:
             comment = o_tree['comment']
@@ -83,7 +124,7 @@ class CfgYamlReader(object):
 
         # Is a valid type?
         if not RegOutput.is_registered(otype):
-            config_error("Unknown output type: `{}`".format(otype))
+            raise KiPlotConfigurationError("Unknown output type: `{}`".format(otype))
         # Load it
         logger.debug("Pre-parsing output options for "+name_type)
         o_out = RegOutput.get_class_for(otype)()
@@ -116,79 +157,85 @@ class CfgYamlReader(object):
             for o in v:
                 outputs.append(self._parse_output(o))
         else:
-            config_error("`outputs` must be a list")
+            raise KiPlotConfigurationError("`outputs` must be a list")
         return outputs
 
-    def _parse_variant(self, o_tree, kind, reg_class):
+    def _parse_variant_or_filter(self, o_tree, kind, reg_class):
         kind_f = kind[0].upper()+kind[1:]
         try:
             name = str(o_tree['name'])
             if not name:
                 raise KeyError
         except KeyError:
-            config_error(kind_f+" needs a name in: "+str(o_tree))
+            raise KiPlotConfigurationError(kind_f+" needs a name in: "+str(o_tree))
         try:
             otype = o_tree['type']
             if not otype:
                 raise KeyError
         except KeyError:
-            config_error(kind_f+" `"+name+"` needs a type")
+            raise KiPlotConfigurationError(kind_f+" `"+name+"` needs a type")
         # Is a valid type?
         if not reg_class.is_registered(otype):
-            config_error("Unknown {} type: `{}`".format(kind, otype))
+            raise KiPlotConfigurationError("Unknown {} type: `{}`".format(kind, otype))
         # Load it
         name_type = "`"+name+"` ("+otype+")"
         logger.debug("Parsing "+kind+" "+name_type)
         o_var = reg_class.get_class_for(otype)()
         o_var.set_tree(o_tree)
-        try:
-            o_var.config(None)
-        except KiPlotConfigurationError as e:
-            config_error("In section `"+name_type+"`: "+str(e))
+        o_var.name = name
+        o_var._name_type = name_type
+        # Don't configure it yet, wait until we finish loading (could be an import)
         return o_var
 
     def _parse_variants(self, v):
         variants = {}
         if isinstance(v, list):
             for o in v:
-                o_var = self._parse_variant(o, 'variant', RegVariant)
+                o_var = self._parse_variant_or_filter(o, 'variant', RegVariant)
                 variants[o_var.name] = o_var
         else:
-            config_error("`variants` must be a list")
+            raise KiPlotConfigurationError("`variants` must be a list")
         return variants
 
     def _parse_filters(self, v):
         filters = {}
         if isinstance(v, list):
             for o in v:
-                o_fil = self._parse_variant(o, 'filter', RegFilter)
+                o_fil = self._parse_variant_or_filter(o, 'filter', RegFilter)
+                self.configure_variant_or_filter(o_fil)
                 filters[o_fil.name] = o_fil
         else:
-            config_error("`filters` must be a list")
+            raise KiPlotConfigurationError("`filters` must be a list")
         return filters
 
-    def _parse_preflight(self, pf):
+    def _parse_preflights(self, pf):
         logger.debug("Parsing preflight options: {}".format(pf))
         if not isinstance(pf, dict):
-            config_error("Incorrect `preflight` section")
+            raise KiPlotConfigurationError("Incorrect `preflight` section")
 
+        preflights = []
         for k, v in pf.items():
             if not BasePreFlight.is_registered(k):
-                config_error("Unknown preflight: `{}`".format(k))
+                raise KiPlotConfigurationError("Unknown preflight: `{}`".format(k))
             try:
                 logger.debug("Parsing preflight "+k)
                 o_pre = BasePreFlight.get_class_for(k)(k, v)
             except KiPlotConfigurationError as e:
-                config_error("In preflight '"+k+"': "+str(e))
-            BasePreFlight.add_preflight(o_pre)
+                raise KiPlotConfigurationError("In preflight '"+k+"': "+str(e))
+            preflights.append(o_pre)
+        return preflights
 
     def _parse_global(self, gb):
         """ Get global options """
         logger.debug("Parsing global options: {}".format(gb))
         if not isinstance(gb, dict):
-            config_error("Incorrect `global` section (must be a dict)")
+            raise KiPlotConfigurationError("Incorrect `global` section (must be a dict)")
         if self.imported_globals:
-            gb.update(self.imported_globals)
+            if self.imported_global_has_less_priority:
+                update_dict(self.imported_globals, gb)
+                gb = self.imported_globals
+            else:
+                update_dict(gb, self.imported_globals)
             logger.debug("Global options + imported: {}".format(gb))
         # Parse all keys inside it
         glb = GS.class_for_global_opts()
@@ -196,13 +243,13 @@ class CfgYamlReader(object):
         try:
             glb.config(None)
         except KiPlotConfigurationError as e:
-            config_error("In `global` section: "+str(e))
+            raise KiPlotConfigurationError("In `global` section: "+str(e))
 
     @staticmethod
     def _config_error_import(fname, error):
         if fname is None:
             fname = '*unnamed*'
-        config_error('{} in {} import'.format(error, fname))
+        raise KiPlotConfigurationError('{} in {} import'.format(error, fname))
 
     @staticmethod
     def _parse_import_items(kind, fname, value):
@@ -222,15 +269,15 @@ class CfgYamlReader(object):
             return values
         CfgYamlReader._config_error_import(fname, '`{}` must be a string or a list ({})'.format(kind, str(v)))
 
-    def _parse_import_outputs(self, outs, explicit_outs, fn_rel, data):
+    def _parse_import_outputs(self, outs, explicit_outs, fn_rel, data, imported):
+        sel_outs = []
         if (outs is None or len(outs) > 0) and 'outputs' in data:
-            i_outs = self._parse_outputs(data['outputs'])
+            i_outs = imported.outputs+self._parse_outputs(data['outputs'])
             if outs is not None:
-                sel_outs = []
                 for o in i_outs:
                     if o.name in outs:
                         sel_outs.append(o)
-                        outs.remove(o)
+                        outs.remove(o.name)
                 for o in outs:
                     logger.warning(W_UNKOUT+"can't import `{}` output from `{}` (missing)".format(o, fn_rel))
             else:
@@ -238,19 +285,38 @@ class CfgYamlReader(object):
             if len(sel_outs) == 0:
                 logger.warning(W_NOOUTPUTS+"No outputs found in `{}`".format(fn_rel))
             else:
-                try:
-                    RegOutput.add_outputs(sel_outs, fn_rel)
-                except KiPlotConfigurationError as e:
-                    config_error(str(e))
                 logger.debug('Outputs loaded from `{}`: {}'.format(fn_rel, list(map(lambda c: c.name, sel_outs))))
         if outs is None and explicit_outs and 'outputs' not in data:
             logger.warning(W_NOOUTPUTS+"No outputs found in `{}`".format(fn_rel))
+        return sel_outs
 
-    def _parse_import_filters(self, filters, explicit_fils, fn_rel, data):
+    def _parse_import_preflights(self, pre, explicit_pres, fn_rel, data, imported):
+        sel_pres = []
+        if (pre is None or len(pre) > 0) and 'preflight' in data:
+            i_pres = imported.preflights+self._parse_preflights(data['preflight'])
+            if pre is not None:
+                for p in i_pres:
+                    if p._name in pre:
+                        sel_pres.append(p)
+                        pre.remove(p._name)
+                for p in pre:
+                    logger.warning(W_UNKOUT+"can't import `{}` preflight from `{}` (missing)".format(p, fn_rel))
+            else:
+                sel_pres = i_pres
+            if len(sel_pres) == 0:
+                logger.warning(W_NOPREFLIGHTS+"No preflights found in `{}`".format(fn_rel))
+            else:
+                logger.debug('Preflights loaded from `{}`: {}'.format(fn_rel, list(map(lambda c: c._name, sel_pres))))
+        if pre is None and explicit_pres and 'preflight' not in data:
+            logger.warning(W_NOPREFLIGHTS+"No preflights found in `{}`".format(fn_rel))
+        return sel_pres
+
+    def _parse_import_filters(self, filters, explicit_fils, fn_rel, data, imported):
+        sel_fils = {}
         if (filters is None or len(filters) > 0) and 'filters' in data:
-            i_fils = self._parse_filters(data['filters'])
+            imported.filters.update(self._parse_filters(data['filters']))
+            i_fils = imported.filters
             if filters is not None:
-                sel_fils = {}
                 for f in filters:
                     if f in i_fils:
                         sel_fils[f] = i_fils[f]
@@ -261,16 +327,17 @@ class CfgYamlReader(object):
             if len(sel_fils) == 0:
                 logger.warning(W_NOFILTERS+"No filters found in `{}`".format(fn_rel))
             else:
-                RegOutput.add_filters(sel_fils)
                 logger.debug('Filters loaded from `{}`: {}'.format(fn_rel, sel_fils.keys()))
         if filters is None and explicit_fils and 'filters' not in data:
             logger.warning(W_NOFILTERS+"No filters found in `{}`".format(fn_rel))
+        return sel_fils
 
-    def _parse_import_variants(self, vars, explicit_vars, fn_rel, data):
+    def _parse_import_variants(self, vars, explicit_vars, fn_rel, data, imported):
+        sel_vars = {}
         if (vars is None or len(vars) > 0) and 'variants' in data:
-            i_vars = self._parse_variants(data['variants'])
+            imported.variants.update(self._parse_variants(data['variants']))
+            i_vars = imported.variants
             if vars is not None:
-                sel_vars = {}
                 for f in vars:
                     if f in i_vars:
                         sel_vars[f] = i_vars[f]
@@ -281,18 +348,21 @@ class CfgYamlReader(object):
             if len(sel_vars) == 0:
                 logger.warning(W_NOVARIANTS+"No variants found in `{}`".format(fn_rel))
             else:
-                RegOutput.add_variants(sel_vars)
                 logger.debug('Variants loaded from `{}`: {}'.format(fn_rel, sel_vars.keys()))
         if vars is None and explicit_vars and 'variants' not in data:
             logger.warning(W_NOVARIANTS+"No variants found in `{}`".format(fn_rel))
+        return sel_vars
 
-    def _parse_import_globals(self, globals, explicit_globals, fn_rel, data):
+    def _parse_import_globals(self, globals, explicit_globals, fn_rel, data, imported):
+        sel_globals = {}
         if (globals is None or len(globals) > 0) and 'global' in data:
             i_globals = data['global']
             if not isinstance(i_globals, dict):
-                config_error("Incorrect `global` section (must be a dict), while importing from {}".format(fn_rel))
+                raise KiPlotConfigurationError("Incorrect `global` section (must be a dict), while importing from {}".
+                                               format(fn_rel))
+            update_dict(imported.globals, i_globals)
+            i_globals = imported.globals
             if globals is not None:
-                sel_globals = {}
                 for f in globals:
                     if f in i_globals:
                         sel_globals[f] = i_globals[f]
@@ -303,18 +373,30 @@ class CfgYamlReader(object):
             if len(sel_globals) == 0:
                 logger.warning(W_NOGLOBALS+"No globals found in `{}`".format(fn_rel))
             else:
-                self.imported_globals.update(sel_globals)
                 logger.debug('Globals loaded from `{}`: {}'.format(fn_rel, sel_globals.keys()))
         if globals is None and explicit_globals and 'global' not in data:
             logger.warning(W_NOGLOBALS+"No globals found in `{}`".format(fn_rel))
+        return sel_globals
 
-    def _parse_import(self, imp, name):
+    def configure_variant_or_filter(self, o_var):
+        o_var.config(None)
+
+    def configure_variants(self, variants):
+        logger.debug('Configuring variants')
+        for o_var in variants.values():
+            self.configure_variant_or_filter(o_var)
+
+    def _parse_import(self, imp, name, apply=True, depth=0):
         """ Get imports """
         logger.debug("Parsing imports: {}".format(imp))
+        depth += 1
+        if depth > 20:
+            raise KiPlotConfigurationError("Import depth greater than 20, make sure this isn't an infinite loop")
         if not isinstance(imp, list):
-            config_error("Incorrect `import` section (must be a list)")
+            raise KiPlotConfigurationError("Incorrect `import` section (must be a list)")
         # Import the files
         dir = os.path.dirname(os.path.abspath(name))
+        all_collected = CollectedImports()
         for entry in imp:
             if isinstance(entry, str):
                 fn = entry
@@ -322,21 +404,26 @@ class CfgYamlReader(object):
                 filters = []
                 vars = []
                 globals = []
+                pre = []
                 explicit_outs = True
                 explicit_fils = False
                 explicit_vars = False
                 explicit_globals = False
+                explicit_pres = False
             elif isinstance(entry, dict):
-                fn = outs = filters = vars = globals = None
-                explicit_outs = explicit_fils = explicit_vars = explicit_globals = False
+                fn = outs = filters = vars = globals = pre = None
+                explicit_outs = explicit_fils = explicit_vars = explicit_globals = explicit_pres = False
                 for k, v in entry.items():
                     if k == 'file':
                         if not isinstance(v, str):
-                            config_error("`import.file` must be a string ({})".format(str(v)))
+                            raise KiPlotConfigurationError("`import.file` must be a string ({})".format(str(v)))
                         fn = v
                     elif k == 'outputs':
                         outs = self._parse_import_items(k, fn, v)
                         explicit_outs = True
+                    elif k == 'preflights':
+                        pre = self._parse_import_items(k, fn, v)
+                        explicit_pres = True
                     elif k == 'filters':
                         filters = self._parse_import_items(k, fn, v)
                         explicit_fils = True
@@ -349,35 +436,71 @@ class CfgYamlReader(object):
                     else:
                         self._config_error_import(fn, "unknown import entry `{}`".format(str(v)))
                 if fn is None:
-                    config_error("`import` entry without `file` ({})".format(str(entry)))
+                    raise KiPlotConfigurationError("`import` entry without `file` ({})".format(str(entry)))
             else:
-                config_error("`import` items must be strings or dicts ({})".format(str(entry)))
+                raise KiPlotConfigurationError("`import` items must be strings or dicts ({})".format(str(entry)))
+            fn = os.path.expandvars(os.path.expanduser(fn))
             if not os.path.isabs(fn):
                 fn = os.path.join(dir, fn)
             if not os.path.isfile(fn):
-                config_error("missing import file `{}`".format(fn))
+                raise KiPlotConfigurationError("missing import file `{}`".format(fn))
             fn_rel = os.path.relpath(fn)
             data = self.load_yaml(open(fn))
+            if 'import' in data:
+                # Do a recursive import
+                imported = self._parse_import(data['import'], fn, apply=False, depth=depth)
+            else:
+                # Nothing to import, start fresh
+                imported = CollectedImports()
+            # Parse and filter all stuff, add them to all_collected
             # Outputs
-            self._parse_import_outputs(outs, explicit_outs, fn_rel, data)
+            all_collected.outputs.extend(self._parse_import_outputs(outs, explicit_outs, fn_rel, data, imported))
+            # Preflights
+            all_collected.preflights.extend(self._parse_import_preflights(pre, explicit_pres, fn_rel, data, imported))
             # Filters
-            self._parse_import_filters(filters, explicit_fils, fn_rel, data)
+            all_collected.filters.update(self._parse_import_filters(filters, explicit_fils, fn_rel, data, imported))
             # Variants
-            self._parse_import_variants(vars, explicit_vars, fn_rel, data)
+            all_collected.variants.update(self._parse_import_variants(vars, explicit_vars, fn_rel, data, imported))
             # Globals
-            self._parse_import_globals(globals, explicit_globals, fn_rel, data)
+            update_dict(all_collected.globals, self._parse_import_globals(globals, explicit_globals, fn_rel, data, imported))
+        if apply:
+            # This is the main import (not a recursive one) apply the results
+            RegOutput.add_filters(all_collected.filters)
+            self.configure_variants(all_collected.variants)
+            RegOutput.add_variants(all_collected.variants)
+            self.imported_globals = all_collected.globals
+            BasePreFlight.add_preflights(all_collected.preflights)
+            RegOutput.add_outputs(all_collected.outputs, fn_rel)
+        return all_collected
 
     def load_yaml(self, fstream):
+        if GS.cli_defines:
+            # Load the file to memory so we can preprocess it
+            content = fstream.read()
+            logger.debug('Applying preprocessor definitions')
+            # Replace all
+            for k, v in GS.cli_defines.items():
+                key = '@'+k+'@'
+                logger.debugl(2, '- Replacing {} -> {}'.format(key, v))
+                content = content.replace(key, v)
+            # Create an stream from the string
+            fstream = io.StringIO(content)
         try:
             data = yaml.safe_load(fstream)
         except yaml.YAMLError as e:
-            config_error("Error loading YAML "+str(e))
+            raise KiPlotConfigurationError("Error loading YAML "+str(e))
         # Accept `globals` for `global`
         if 'globals' in data and 'global' not in data:
             data['global'] = data['globals']
             del data['globals']
         return data
 
+    def _check_invalid_in_kibot(self, main_sec):
+        defined_in_kibot = set(main_sec.keys())
+        invalid_in_kibot = defined_in_kibot-VALID_KIBOT_SEC
+        for k in invalid_in_kibot:
+            raise KiPlotConfigurationError('Unknown option `{}` in kibot/kiplot.'.format(k))
+
     def read(self, fstream):
         """
         Read a file object into a config object
@@ -385,43 +508,54 @@ class CfgYamlReader(object):
         :param fstream: file stream of a config YAML file
         """
         data = self.load_yaml(fstream)
-        # List of outputs
-        version = None
-        globals_found = False
-        # Analyze each section
-        for k, v in data.items():
-            # logger.debug('{} {}'.format(k, v))
-            if k == 'kiplot' or k == 'kibot':
-                version = self._check_version(v)
-            elif k == 'preflight':
-                self._parse_preflight(v)
-            elif k == 'global':
-                self._parse_global(v)
-                globals_found = True
-            elif k == 'import':
-                self._parse_import(v, fstream.name)
-            elif k == 'variants':
-                RegOutput.add_variants(self._parse_variants(v))
-            elif k == 'filters':
-                RegOutput.add_filters(self._parse_filters(v))
-            elif k == 'outputs':
-                try:
-                    RegOutput.add_outputs(self._parse_outputs(v))
-                except KiPlotConfigurationError as e:
-                    config_error(str(e))
-            else:
-                config_error('Unknown section `{}` in config.'.format(k))
-        if version is None:
-            config_error("YAML config needs `kibot.version`.")
+        # Analyze the version
+        # Currently just checks for v1
+        v1 = data.get('kiplot', None)
+        v2 = data.get('kibot', None)
+        if v1 and v2:
+            raise KiPlotConfigurationError("Use `kibot` or `kiplot` but not both.")
+        if not v1 and not v2:
+            raise KiPlotConfigurationError("YAML config needs `kibot.version`.")
+        main_sec = v1 or v2
+        self._check_version(main_sec)
+        self._check_globals_priority(main_sec)
+        self._check_invalid_in_kibot(main_sec)
+        # Look for imports
+        v1 = data.get('import', None)
+        if v1:
+            self._parse_import(v1, fstream.name)
+        # Look for globals
         # If no globals defined initialize them with default values
-        if not globals_found:
-            self._parse_global({})
+        self._parse_global(data.get('global', {}))
+        # Look for filters
+        v1 = data.get('filters', None)
+        if v1:
+            RegOutput.add_filters(self._parse_filters(v1))
+        # Look for variants
+        v1 = data.get('variants', None)
+        if v1:
+            variants = self._parse_variants(v1)
+            self.configure_variants(variants)
+            RegOutput.add_variants(variants)
         # Solve the global variant
         if GS.global_variant:
             try:
                 GS.solved_global_variant = RegOutput.check_variant(GS.global_variant)
             except KiPlotConfigurationError as e:
-                config_error("In global section: "+str(e))
+                raise KiPlotConfigurationError("In global section: "+str(e))
+        # Look for preflights
+        v1 = data.get('preflight', None)
+        if v1:
+            BasePreFlight.add_preflights(self._parse_preflights(v1))
+        # Look for outputs
+        v1 = data.get('outputs', None)
+        if v1:
+            RegOutput.add_outputs(self._parse_outputs(v1))
+        # Report invalid sections (the first we find)
+        defined_sections = set(data.keys())
+        invalid_sections = defined_sections-VALID_SECTIONS
+        for k in invalid_sections:
+            raise KiPlotConfigurationError('Unknown section `{}` in config.'.format(k))
         # Ok, now we have all the outputs loaded, so we can apply the disable_run_by_default
         for name in self.no_run_by_default:
             o = RegOutput.get_output(name)
@@ -557,6 +691,19 @@ def print_preflights_help():
             print_output_options(n, options, 2)
 
 
+def print_variants_help():
+    from .var_base import BaseVariant
+    vars = BaseVariant.get_registered()
+    logger.debug('{} supported variants'.format(len(vars)))
+    print('Supported variants:\n')
+    for n, o in OrderedDict(sorted(vars.items())).items():
+        help = o.__doc__
+        if help is None:
+            help = 'Undocumented'
+        print('- `{}`: {}.'.format(n, help.strip()))
+        print_output_options(n, o, 2)
+
+
 def print_filters_help():
     filters = RegFilter.get_registered()
     logger.debug('{} supported filters'.format(len(filters)))
@@ -717,6 +864,32 @@ class MyEncoder(json.JSONEncoder):
         return o.__dict__
 
 
+def print_dep_comments(dep):
+    if len(dep.comments) == 1:
+        print('- Note: '+dep.comments[0])
+    elif len(dep.comments) > 1:
+        print('- Notes:')
+        for comment in dep.comments:
+            print('  - '+comment)
+
+
+def compose_version(version, max_version):
+    ver = ' v'+'.'.join(map(str, version)) if version else ''
+    ver += ' (<'+'.'.join(map(str, max_version))+')' if max_version else ''
+    return ver
+
+
+def print_needed(needed):
+    if needed:
+        if len(needed) == 1:
+            if needed[0] == 'general use':
+                print('- Mandatory')
+            else:
+                print('- Mandatory for '+needed[0])
+        else:
+            print('- Mandatory for: '+', '.join(sorted(needed)))
+
+
 def print_dependencies(markdown=True, jsn=False):
     # Compute the importance of each dependency
     for dep in RegDependency.get_registered().values():
@@ -735,20 +908,31 @@ def print_dependencies(markdown=True, jsn=False):
     # Now print them sorted by importance (and by name as a second criteria)
     for name, dep in sorted(sorted(RegDependency.get_registered().items(), key=lambda x: x[0].lower()),   # noqa C414
                             key=lambda x: x[1].importance, reverse=True):
-        dtype = 'python module' if dep.is_python else 'tool'
-        is_pypi_dep = ' (PyPi dependency)' if dep.pypi_name.lower() in __pypi_deps__ else ''
         deb = ''
         if markdown:
+            dtype = PY_LOGO if dep.is_python else TOOL_LOGO
+            is_pypi_dep = PYPI_LOGO if dep.pypi_name.lower() in __pypi_deps__ else ''
+            has_dowloader = ' '+AUTO_DOWN if dep.downloader is not None else ''
             if dep.is_python:
                 url = 'https://pypi.org/project/{}/'.format(name)
+                if is_pypi_dep:
+                    is_pypi_dep = ' [{}]({})'.format(is_pypi_dep, url)
             else:
                 url = dep.url
             name = '[**{}**]({})'.format(name, url)
+            dtype = ' [{}]({})'.format(dtype, url)
             if dep.in_debian:
-                deb = ' [Debian](https://packages.debian.org/bullseye/{})'.format(dep.deb_package)
+                deb = ' [{}](https://packages.debian.org/bullseye/{})'.format(DEB_LOGO, dep.deb_package)
+        else:
+            dtype = ' (Python module)' if dep.is_python else ' (Tool)'
+            is_pypi_dep = ' (PyPi dependency)' if dep.pypi_name.lower() in __pypi_deps__ else ''
+            has_dowloader = ' (Auto-download)' if dep.downloader is not None else ''
+            if dep.in_debian:
+                deb = ' (Debian: {})'.format(dep.deb_package)
         needed = []
         optional = []
         version = None
+        max_version = None
         for r in dep.roles:
             if r.mandatory:
                 needed.append(global2human(r.output))
@@ -756,18 +940,11 @@ def print_dependencies(markdown=True, jsn=False):
                 optional.append(r)
             if r.version and (version is None or r.version > version):
                 version = r.version
-        ver = ''
-        if version:
-            ver = 'v'+'.'.join(map(str, version))+' '
-        print("{} {}({}){}{}".format(name, ver, dtype, is_pypi_dep, deb))
-        if needed:
-            if len(needed) == 1:
-                if needed[0] == 'general use':
-                    print('- Mandatory')
-                else:
-                    print('- Mandatory for '+needed[0])
-            else:
-                print('- Mandatory for: '+', '.join(sorted(needed)))
+            if r.max_version and (max_version is None or r.max_version < max_version):
+                max_version = r.max_version
+        ver = compose_version(version, max_version)
+        print(name+ver+dtype+is_pypi_dep+deb+has_dowloader)
+        print_needed(needed)
         if optional:
             if len(optional) == 1:
                 o = optional[0]
@@ -780,4 +957,5 @@ def print_dependencies(markdown=True, jsn=False):
                     if o.version:
                         ver = ' (v'+'.'.join(map(str, o.version))+')'
                     print('  - {} for {}{}'.format(o.desc, global2human(o.output), ver))
+        print_dep_comments(dep)
         print()
