@@ -4,13 +4,15 @@
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
 import os
+from .error import KiPlotConfigurationError
 from .gs import GS
 from .optionable import Optionable
 from .kicad.config import expand_env
 from .macros import macros, document  # noqa: F401
-from .pre_filters import FiltersOptions
+from .pre_filters import FiltersOptions, FilterOptionsKiBot
 from .log import get_logger, set_filters
 from .misc import W_MUSTBEINT
+from .kicad.config import KiConf
 from .kicad.sexpdata import load, SExpData, sexp_iter, Symbol
 from .kicad.v6_sch import PCBLayer
 
@@ -42,8 +44,8 @@ class Environment(Optionable):
         if self.symbols:
             defs['KICAD_SYMBOL_DIR'] = self.symbols
         if self.footprints:
-            defs['KICAD_FOOTPRINT_DIR'] = self.symbols
-            defs['KISYSMOD'] = self.symbols
+            defs['KICAD_FOOTPRINT_DIR'] = self.footprints
+            defs['KISYSMOD'] = self.footprints
         if self.models_3d:
             defs['KISYS3DMOD'] = self.models_3d
         if self.templates:
@@ -55,7 +57,7 @@ class Environment(Optionable):
         if self.symbols:
             defs['KICAD6_SYMBOL_DIR'] = self.symbols
         if self.footprints:
-            defs['KICAD6_FOOTPRINT_DIR'] = self.symbols
+            defs['KICAD6_FOOTPRINT_DIR'] = self.footprints
         if self.models_3d:
             defs['KICAD6_3DMODEL_DIR'] = self.models_3d
         if self.templates:
@@ -68,7 +70,7 @@ class Environment(Optionable):
     def config(self, parent):
         super().config(parent)
         defs = {}
-        if GS.ki5():
+        if GS.ki5:
             self.define_k5_vars(defs)
         else:
             self.define_k6_vars(defs)
@@ -87,11 +89,41 @@ class Environment(Optionable):
                 os.environ[n] = v
 
 
+class KiCadAlias(Optionable):
+    """ KiCad alias (for 3D models) """
+    def __init__(self):
+        super().__init__()
+        self._unkown_is_error = True
+        with document:
+            self.name = ''
+            """ Name of the alias """
+            self.variable = None
+            """ {name} """
+            self.alias = None
+            """ {name} """
+            self.value = ''
+            """ Path to the 3D model """
+            self.text = None
+            """ {value} """
+
+    def config(self, parent):
+        super().config(parent)
+        if not self.name:
+            raise KiPlotConfigurationError("Missing variable name ({})".format(str(self._tree)))
+
+
 class Globals(FiltersOptions):
     """ Global options """
     def __init__(self):
         super().__init__()
         with document:
+            self.aliases_for_3d_models = KiCadAlias
+            """ [list(dict)] List of aliases for the 3D models (KiCad 6).
+                KiCad stores 3D aliases with the user settings, not locally.
+                This makes impossible to create self contained projects.
+                You can define aliases here to workaround this problem.
+                The values defined here has precedence over the KiCad configuration.
+                Related to https://gitlab.com/kicad/code/kicad/-/issues/3792 """
             self.castellated_pads = False
             """ Has the PCB castelletad pads?
                 KiCad 6: you should set this in the Board Setup -> Board Finish -> Has castellated pads """
@@ -100,6 +132,12 @@ class Globals(FiltersOptions):
             self.copper_thickness = 35
             """ [number|string] Copper thickness in micrometers (1 Oz is 35 micrometers).
                 KiCad 6: you should set this in the Board Setup -> Physical Stackup """
+            self.cross_footprints_for_dnp = True
+            """ Draw a cross for excluded components in the `Fab` layer """
+            self.cross_no_body = False
+            """ Cross components even when they don't have a body. Only for KiCad 6 """
+            self.csv_accept_no_ref = False
+            """ Accept aggregating CSV files without references (Experimental) """
             self.date_format = '%Y-%m-%d'
             """ Format used for the day we started the script.
                 Is also used for the PCB/SCH date formatting when `time_reformat` is enabled (default behavior).
@@ -107,7 +145,15 @@ class Globals(FiltersOptions):
             self.date_time_format = '%Y-%m-%d_%H-%M-%S'
             """ Format used for the PCB and schematic date when using the file timestamp. Uses the `strftime` format """
             self.dir = ''
-            """ Default pattern for the output directories """
+            """ Default pattern for the output directories. It also applies to the preflights, unless
+                `use_dir_for_preflights` is disabled """
+            self.disable_3d_alias_as_env = False
+            """ Disable the use of environment and text variables as 3D models aliases """
+            self.drc_exclusions_workaround = False
+            """ KiCad 6 introduced DRC exclusions. They are stored in the project but ignored by the Python API.
+                This is reported as bug number 11562 (https://gitlab.com/kicad/code/kicad/-/issues/11562).
+                If you really need exclusions enable this option, this will use the GUI version of the DRC (slower).
+                Current KiCad version is 6.0.7 and the bug is still there """
             self.drill_size_increment = 0.05
             """ This is the difference between drill tools in millimeters.
                 A manufacturer with 0.05 of increment has drills for 0.1, 0.15, 0.2, 0.25, etc. """
@@ -123,6 +169,8 @@ class Globals(FiltersOptions):
                 For more information consult: https://www.eurocircuits.com/pcb-design-guidelines/drilled-holes/ """
             self.field_3D_model = '_3D_model'
             """ Name for the field controlling the 3D models used for a component """
+            self.hide_excluded = False
+            """ Default value for the `hide_excluded` option of various PCB outputs """
             self.kiauto_time_out_scale = 0.0
             """ Time-out multiplier for KiAuto operations """
             self.kiauto_wait_start = 0
@@ -140,6 +188,18 @@ class Globals(FiltersOptions):
             self.pcb_material = 'FR4'
             """ PCB core material. Currently used for documentation and to choose default colors.
                 Currently known are FR1 to FR5 """
+            self.remove_solder_paste_for_dnp = True
+            """ When applying filters and variants remove the solder paste for components that won't be included """
+            self.remove_adhesive_for_dnp = True
+            """ When applying filters and variants remove the adhesive (glue) for components that won't be included """
+            self.restore_project = False
+            """ Restore the KiCad project after execution.
+                Note that this option will undo operations like `set_text_variables` """
+            self.set_text_variables_before_output = False
+            """ Run the `set_text_variables` preflight before running each output that involves variants.
+                This can be used when a text variable uses the variant and you want to create more than
+                one variant in the same run. Note that this could be slow because it forces a board
+                reload each time you run an output that uses variants """
             self.silk_screen_color = 'white'
             """ Color for the markings. Currently used for documentation and to choose default colors.
                 KiCad 6: you should set this in the Board Setup -> Physical Stackup.
@@ -167,6 +227,8 @@ class Globals(FiltersOptions):
                 This assumes you let KiCad fill this value and hence the time is in ISO format (YY-MM-DD) """
             self.units = ''
             """ [millimeters,inches,mils] Default units. Affects `position` and `bom` outputs. Also KiCad 6 dimensions """
+            self.use_dir_for_preflights = True
+            """ Use the global `dir` as subdir for the preflights """
             self.variant = ''
             """ Default variant to apply to all outputs """
             self.out_dir = ''
@@ -179,6 +241,7 @@ class Globals(FiltersOptions):
                 The KIPRJMOD is also available for expansion """
         self.set_doc('filters', " [list(dict)] KiBot warnings to be ignored ")
         self._filter_what = 'KiBot warnings'
+        self.filters = FilterOptionsKiBot
         self._unkown_is_error = True
         self._error_context = 'global '
 
@@ -280,7 +343,7 @@ class Globals(FiltersOptions):
             logger.debug("- Copper thickness: "+self.copper_thickness)
 
     def config(self, parent):
-        if GS.ki6() and GS.pcb_file and os.path.isfile(GS.pcb_file):
+        if GS.ki6 and GS.pcb_file and os.path.isfile(GS.pcb_file):
             self.get_stack_up()
         super().config(parent)
         # Transfer options to the GS globals
@@ -312,6 +375,14 @@ class Globals(FiltersOptions):
             if not GS.global_silk_screen_color_bottom:
                 GS.global_silk_screen_color_bottom = GS.global_silk_screen_color
         set_filters(self.unparsed)
+        # 3D models aliases
+        if isinstance(self.aliases_for_3d_models, list):
+            KiConf.init(GS.pcb_file or GS.sch_file)
+            logger.debug('Adding 3D models aliases from global config')
+            for alias in self.aliases_for_3d_models:
+                KiConf.aliases_3D[alias.name] = alias.value
+                logger.debugl(1, '- {}={}'.format(alias.name, alias.value))
+            logger.debugl(1, 'Finished adding aliases')
 
 
 logger = get_logger(__name__)
