@@ -686,6 +686,8 @@ class SymLib(object):
             f = LibLineReader(fh, file)
             line = f.get_line()
             if not line.startswith('EESchema-LIBRARY'):
+                if file.endswith('kicad_sym'):
+                    raise SchLibError('Mixing KiCad 5 and KiCad 6 files is not allowed', line, f)
                 raise SchLibError('Missing library signature', line, f)
             line = f.get_line()
             translate = {k.replace(':', '_'): k for k, v in needed.items() if v is None} if lib_alias is None else None
@@ -882,6 +884,7 @@ class SchematicComponent(object):
         self.field_ref = ''
         self.value = ''
         self.footprint = ''
+        self.footprint_lib = None
         self.datasheet = ''
         self.desc = ''
         self.fields = []
@@ -896,12 +899,20 @@ class SchematicComponent(object):
         self.footprint_rot = 0.0
         self.footprint_x = self.footprint_y = 0
         self.footprint_w = self.footprint_h = 0
+        self.has_pcb_info = False
         self.qty = 1
         self.annotation_error = False
         # KiCad 5 PCB flags (mutually exclusive)
         self.smd = False
         self.virtual = False
         self.tht = False
+        # KiCad 6 SCH flags
+        self.in_bom = True
+        self.on_board = True
+        # KiCad 6 PCB flags
+        self.in_bom_pcb = True
+        self.in_pos = True
+        self.in_pcb_only = False
 
     def get_field_value(self, field):
         field = field.lower()
@@ -1040,6 +1051,15 @@ class SchematicComponent(object):
             return '{} ({})'.format(ref, self.name)
         return '{} ({} {})'.format(ref, self.name, self.value)
 
+    def split_ref(self, f=None):
+        m = SchematicComponent.ref_re.match(self.ref)
+        if not m:
+            if f:
+                raise SchFileError('Malformed component reference', self.ref, f)
+            else:
+                raise SchError('Malformed component reference `{}`'.format(self.ref))
+        self.ref_prefix, self.ref_suffix = m.groups()
+
     @staticmethod
     def load(f, project, sheet_path, sheet_path_h, libs, fields, fields_lc):
         # L lib:name reference
@@ -1132,10 +1152,7 @@ class SchematicComponent(object):
             logger.warning(W_NOANNO + 'Component {} is not annotated'.format(comp))
             comp.annotation_error = True
         # Separate the reference in its components
-        m = SchematicComponent.ref_re.match(comp.ref)
-        if not m:
-            raise SchFileError('Malformed component reference', comp.ref, f)
-        comp.ref_prefix, comp.ref_suffix = m.groups()
+        comp.split_ref(f)
         # Location in the project
         comp.sheet_path = sheet_path
         comp.sheet_path_h = sheet_path_h
@@ -1831,6 +1848,18 @@ class Schematic(object):
             f.write(')\n')
         return fname
 
+    def get_title(self):
+        return self.title_block.get('Title', '')
+
+    def set_title(self, title):
+        """ Used only to save a variant """
+        old_title = self.title_block.get('Title')
+        if title is None:
+            del self.title_block['Title']
+        else:
+            self.title_block['Title'] = title
+        return old_title
+
     def file_names_variant(self, dest_dir):
         """ Returns a list of file names created by save_variant() """
         fnames = [os.path.join(dest_dir, 'y.lib'),
@@ -1901,7 +1930,7 @@ class Schematic(object):
             SubElement(comp, 'value').text = c.value
             if c.footprint:
                 SubElement(comp, 'footprint').text = c.footprint
-            if len(c.datasheet) and not (self.netlist_version == 'E' and c.datasheet != '~'):
+            if len(c.datasheet) and not (self.netlist_version == 'E' and c.datasheet == '~'):
                 SubElement(comp, 'datasheet').text = c.datasheet
             user_fields = c.get_user_fields()
             if user_fields:
@@ -1949,7 +1978,11 @@ class Schematic(object):
                 continue
             libpart = SubElement(libparts, 'libpart')
             res = k.split(':')
-            if res:
+            cres = len(res)
+            if cres == 1:
+                libpart.set('lib', '')
+                libpart.set('part', res[0])
+            elif cres == 2:
                 libpart.set('lib', res[0])
                 libpart.set('part', res[1])
             if v.alias:
