@@ -21,6 +21,7 @@ from .sexpdata import load, SExpData, Symbol, dumps, Sep
 
 logger = log.get_logger()
 CROSSED_LIB = 'kibot_crossed'
+NO_YES = ['no', 'yes']
 
 
 def _check_is_symbol_list(e, allow_orphan_symbol=()):
@@ -760,8 +761,8 @@ class LibComponent(object):
         self.pin_numbers_hide = None
         self.pin_names_hide = None
         self.pin_names_offset = None
-        self.in_bom = False
-        self.on_board = False
+        self.in_bom = True
+        self.on_board = True
         self.is_power = False
         self.unit = 0
         self.draw = []
@@ -896,15 +897,23 @@ class LibComponent(object):
         for c in range(self.unit_count):
             name = self.name+"_"+str(c+1)
             box = Box()
-            unit_with_graphs = None
+            units_with_graphs = []
+            unit_without_graphs = None
             for unit in self.units:
                 # Unit 0 is part of unit 1
                 if unit.name.startswith(name) or (c == 0 and unit.name.startswith(name0)):
                     box.union(unit.box)
                     if len(unit.draw):
-                        unit_with_graphs = unit
-            if unit_with_graphs:
-                unit_with_graphs.cross_box = box
+                        # We can have more than one drawing. This is used for the alternative version
+                        units_with_graphs.append(unit)
+                    else:
+                        unit_without_graphs = unit
+            if units_with_graphs:
+                for u in units_with_graphs:
+                    u.cross_box = box
+            elif unit_without_graphs is not None and GS.global_cross_no_body:
+                unit_without_graphs.cross_box = box
+            # Note: if unit_without_graphs is None is because we are inside a unit and it doesn't have a sub-unit
 
     def write_cross(s, sdata):
         """ Add the cross drawing """
@@ -939,10 +948,9 @@ class LibComponent(object):
             if s.pin_names_hide is not None:
                 aux.append(Symbol('hide'))
             sdata.append(_symbol('pin_names', aux))
-        if s.in_bom:
-            sdata.append(_symbol('in_bom', [Symbol('yes')]))
-        if s.on_board:
-            sdata.append(_symbol('on_board', [Symbol('yes')]))
+        if s.units:
+            sdata.append(_symbol('in_bom', [Symbol(NO_YES[s.in_bom])]))
+            sdata.append(_symbol('on_board', [Symbol(NO_YES[s.on_board])]))
         sdata.append(Sep())
         # Properties
         for f in s.fields:
@@ -965,8 +973,6 @@ class LibComponent(object):
 class SchematicComponentV6(SchematicComponent):
     def __init__(self):
         super().__init__()
-        self.in_bom = False
-        self.on_board = False
         self.pins = OrderedDict()
         self.unit = 1
         self.unit_specified = False
@@ -980,10 +986,12 @@ class SchematicComponentV6(SchematicComponent):
     def set_ref(self, ref):
         self.ref = ref
         # Separate the reference in its components
-        m = SchematicComponent.ref_re.match(ref)
-        if not m:
-            raise SchError('Malformed component reference `{}`'.format(ref))
-        self.ref_prefix, self.ref_suffix = m.groups()
+        self.split_ref()
+        self.set_field('Reference', ref)
+
+    def set_value(self, value):
+        self.value = value
+        self.set_field('Value', value)
 
     def set_footprint(self, fp):
         res = fp.split(':')
@@ -996,6 +1004,7 @@ class SchematicComponentV6(SchematicComponent):
             self.footprint = res[1]
         else:
             raise SchError('Footprint with more than one colon (`{}`)'.format(fp))
+        self.set_field('Footprint', fp)
 
     @staticmethod
     def get_lib_and_name(comp, i, name):
@@ -1118,14 +1127,11 @@ class SchematicComponentV6(SchematicComponent):
         if self.convert is not None:
             data.append(_symbol('convert', [self.convert]))
         data.append(Sep())
-        if self.in_bom or self.on_board:
-            if self.in_bom:
-                data.append(_symbol('in_bom', [Symbol('yes')]))
-            if self.on_board:
-                data.append(_symbol('on_board', [Symbol('yes')]))
-            if self.fields_autoplaced:
-                data.append(_symbol('fields_autoplaced'))
-            data.append(Sep())
+        data.append(_symbol('in_bom', [Symbol(NO_YES[self.in_bom])]))
+        data.append(_symbol('on_board', [Symbol(NO_YES[self.on_board])]))
+        if self.fields_autoplaced:
+            data.append(_symbol('fields_autoplaced'))
+        data.append(Sep())
         data.extend([_symbol('uuid', [Symbol(self.uuid)]), Sep()])
         for f in self.fields:
             d = f.write()
@@ -1632,7 +1638,7 @@ class SchematicV6(Schematic):
         return [Sep(), Sep(), _symbol('paper', paper_data)]
 
     def write_title_block(self):
-        data = [Sep()]
+        data = []
         if self.title_ori:
             data += [_symbol('title', [self.title_ori]), Sep()]
         if self.date_ori:
@@ -1644,7 +1650,10 @@ class SchematicV6(Schematic):
         for num, val in enumerate(self.comment_ori):
             if val:
                 data += [_symbol('comment', [num+1, val]), Sep()]
-        return [Sep(), Sep(), _symbol('title_block', data)]
+        # If all empty return an empty list
+        if not data:
+            return data
+        return [Sep(), Sep(), _symbol('title_block', [Sep()]+data)]
 
     def write_lib_symbols(self, cross=False):
         data = [Sep()]
@@ -1682,8 +1691,7 @@ class SchematicV6(Schematic):
             sch.append(Sep())
             sch.append(_symbol('uuid', [Symbol(self.uuid)]))
             sch.extend(self.write_paper())
-            if self.title_ori is not None:
-                sch.extend(self.write_title_block())
+            sch.extend(self.write_title_block())
             sch.extend(self.write_lib_symbols(cross))
             # Bus aliases
             _add_items(self.bus_alias, sch)
@@ -1720,11 +1728,12 @@ class SchematicV6(Schematic):
             _add_items_list('sheet_instances', self.sheet_instances, sch)
             # Symbol instances
             # Copy potentially modified data from components
-            for s in self.symbol_instances:
-                comp = s.component
-                s.reference = comp.ref
-                s.value = comp.value
-                s.footprint = comp.footprint_lib+':'+comp.footprint if comp.footprint_lib else comp.footprint
+            if base_sheet == self:
+                for s in self.symbol_instances:
+                    comp = s.component
+                    s.reference = comp.ref
+                    s.value = comp.value
+                    s.footprint = comp.footprint_lib+':'+comp.footprint if comp.footprint_lib else comp.footprint
             _add_items_list('symbol_instances', self.symbol_instances, sch)
             logger.debug('Saving schematic: `{}`'.format(fname))
             # Keep a back-up of existing files
@@ -1757,6 +1766,15 @@ class SchematicV6(Schematic):
         fparts = os.path.splitext(file)
         sch.flat_file = fparts[0]+'_'+str(len(self.sheet_names))+fparts[1]
 
+    def get_title(self):
+        return self.title_ori
+
+    def set_title(self, title):
+        """ Used only to save a variant """
+        old_title = self.title_ori
+        self.title_ori = title
+        return old_title
+
     def load(self, fname, project, parent=None):  # noqa: C901
         """ Load a v6.x KiCad Schematic.
             The caller must be sure the file exists.
@@ -1897,7 +1915,7 @@ class SchematicV6(Schematic):
             # Transfer the instance data
             comp.set_ref(s.reference)
             comp.unit = s.unit
-            comp.value = s.value
+            comp.set_value(s.value)
             comp.set_footprint(s.footprint)
             comp.sheet_path = path
             comp.sheet_path_h = self.path_to_human(path)
