@@ -13,22 +13,25 @@ import os
 import re
 from sys import exit
 from sys import path as sys_path
+import shlex
 from shutil import which
-from subprocess import run, PIPE, Popen
+from subprocess import run, PIPE, STDOUT, Popen, CalledProcessError
 from glob import glob
-from distutils.version import StrictVersion
-from importlib.util import (spec_from_file_location, module_from_spec)
+from importlib.util import spec_from_file_location, module_from_spec
 from collections import OrderedDict
 
 from .gs import GS
 from .registrable import RegOutput
-from .misc import (PLOT_ERROR, MISSING_TOOL, CMD_EESCHEMA_DO, URL_EESCHEMA_DO, CORRUPTED_PCB,
-                   EXIT_BAD_ARGS, CORRUPTED_SCH, EXIT_BAD_CONFIG, WRONG_INSTALL, UI_SMD, UI_VIRTUAL, TRY_INSTALL_CHECK,
-                   MOD_SMD, MOD_THROUGH_HOLE, MOD_VIRTUAL, W_PCBNOSCH, W_NONEEDSKIP, W_WRONGCHAR, name2make, W_TIMEOUT,
-                   W_KIAUTO, W_VARSCH, NO_SCH_FILE, NO_PCB_FILE, W_VARPCB, NO_YAML_MODULE, WRONG_ARGUMENTS)
+from .misc import (PLOT_ERROR, CORRUPTED_PCB, EXIT_BAD_ARGS, CORRUPTED_SCH, version_str2tuple,
+                   EXIT_BAD_CONFIG, WRONG_INSTALL, UI_SMD, UI_VIRTUAL, TRY_INSTALL_CHECK, MOD_SMD, MOD_THROUGH_HOLE,
+                   MOD_VIRTUAL, W_PCBNOSCH, W_NONEEDSKIP, W_WRONGCHAR, name2make, W_TIMEOUT, W_KIAUTO, W_VARSCH,
+                   NO_SCH_FILE, NO_PCB_FILE, W_VARPCB, NO_YAML_MODULE, WRONG_ARGUMENTS, FAILED_EXECUTE,
+                   MOD_EXCLUDE_FROM_POS_FILES, MOD_EXCLUDE_FROM_BOM, MOD_BOARD_ONLY)
 from .error import PlotError, KiPlotConfigurationError, config_error, trace_dump
 from .config_reader import CfgYamlReader
 from .pre_base import BasePreFlight
+from .dep_downloader import register_deps
+import kibot.dep_downloader as dep_downloader
 from .kicad.v5_sch import Schematic, SchFileError, SchError
 from .kicad.v6_sch import SchematicV6
 from .kicad.config import KiConfError
@@ -48,6 +51,16 @@ except ImportError:
     exit(NO_YAML_MODULE)
 
 
+def try_register_deps(mod, name):
+    if mod.__doc__:
+        try:
+            data = yaml.safe_load(mod.__doc__)
+        except yaml.YAMLError as e:
+            logger.error('While loading plug-in `{}`:'.format(name))
+            config_error("Error loading YAML "+str(e))
+        register_deps(name, data)
+
+
 def _import(name, path):
     # Python 3.4+ import mechanism
     spec = spec_from_file_location("kibot."+name, path)
@@ -60,6 +73,7 @@ def _import(name, path):
         logger.error('Make sure you used `--no-compile` if you used pip for installation')
         logger.error('Python path: '+str(sys_path))
         exit(WRONG_INSTALL)
+    try_register_deps(mod, name)
 
 
 def _load_actions(path, load_internals=False):
@@ -80,6 +94,7 @@ def load_actions():
     if actions_loaded:
         return
     actions_loaded = True
+    try_register_deps(dep_downloader, 'global')
     from kibot.mcpyrate import activate
     # activate.activate()
     _load_actions(os.path.abspath(os.path.dirname(__file__)), True)
@@ -97,57 +112,6 @@ def load_actions():
         activate.deactivate()
 
 
-def check_version(command, version):
-    global script_versions
-    if command in script_versions:
-        return
-    cmd = [command, '--version']
-    if not which(command) and not os.access(command, os.X_OK) and command.endswith('.py'):
-        cmd.insert(0, 'python3')
-    logger.debug('Running: '+str(cmd))
-    result = run(cmd, stdout=PIPE, stderr=PIPE, universal_newlines=True)
-    z = re.match(command + r' (\d+\.\d+\.\d+)', result.stdout, re.IGNORECASE)
-    if not z:
-        z = re.search(r'Version: (\d+\.\d+\.\d+)', result.stdout, re.IGNORECASE)
-    if not z:
-        logger.error('Unable to determine ' + command + ' version:\n' +
-                     result.stdout)
-        exit(MISSING_TOOL)
-    res = z.groups()
-    if StrictVersion(res[0]) < StrictVersion(version):
-        logger.error('Wrong version for `'+command+'` ('+res[0]+'), must be ' +
-                     version+' or newer.')
-        exit(MISSING_TOOL)
-    script_versions[command] = res[0]
-
-
-def check_script(cmd, url, version=None):
-    if which(cmd) is None and not os.path.isfile(cmd):
-        logger.error('No `'+cmd+'` command found.\n'
-                     'Please install it, visit: '+url)
-        logger.error(TRY_INSTALL_CHECK)
-        exit(MISSING_TOOL)
-    if version is not None:
-        check_version(cmd, version)
-
-
-def check_eeschema_do():
-    check_script(CMD_EESCHEMA_DO, URL_EESCHEMA_DO, '1.5.4')
-
-
-def search_as_plugin(cmd, names):
-    """ If a command isn't in the path look for it in the KiCad plugins """
-    if which(cmd) is not None:
-        return cmd
-    for dir in GS.kicad_plugins_dirs:
-        for name in names:
-            fname = os.path.join(dir, name, cmd)
-            if os.path.isfile(fname):
-                logger.debug('Using `{}` for `{}` ({})'.format(fname, cmd, name))
-                return fname
-    return cmd
-
-
 def extract_errors(text):
     in_error = in_warning = False
     msg = ''
@@ -176,10 +140,32 @@ def extract_errors(text):
         logger.warning(W_KIAUTO+msg.rstrip())
 
 
+def debug_output(res):
+    if res.stdout:
+        logger.debug('- Output from command: '+res.stdout.decode())
+
+
+def run_command(command, change_to=None, just_raise=False):
+    logger.debug('Executing: '+shlex.join(command))
+    if change_to is not None:
+        logger.debug('- CWD: '+change_to)
+    try:
+        res = run(command, check=True, stdout=PIPE, stderr=STDOUT, cwd=change_to)
+    except CalledProcessError as e:
+        if just_raise:
+            raise
+        logger.error('Running {} returned {}'.format(e.cmd, e.returncode))
+        debug_output(e)
+        exit(FAILED_EXECUTE)
+    debug_output(res)
+    return res.stdout.decode().rstrip()
+
+
 def exec_with_retry(cmd):
-    logger.debug('Executing: '+str(cmd))
+    cmd_str = shlex.join(cmd)
+    logger.debug('Executing: '+cmd_str)
     if GS.debug_level > 2:
-        logger.debug('Command line: '+' '.join(cmd))
+        logger.debug('Command line: '+cmd_str)
     retry = 2
     while retry:
         result = run(cmd, stdout=PIPE, stderr=PIPE, universal_newlines=True)
@@ -227,7 +213,7 @@ def load_board(pcb_file=None):
         board = pcbnew.LoadBoard(pcb_file)
         if BasePreFlight.get_option('check_zone_fills'):
             pcbnew.ZONE_FILLER(board).Fill(board.Zones())
-        if GS.global_units and GS.ki6():
+        if GS.global_units and GS.ki6:
             # In KiCad 6 "dimensions" has units.
             # The default value is DIM_UNITS_MODE_AUTOMATIC.
             # But this has a meaning only in the GUI where you have default units.
@@ -294,36 +280,51 @@ def get_board_comps_data(comps):
     if not GS.pcb_file:
         return
     load_board()
-    comps_hash = {c.ref: c for c in comps}
+    # Each reference could be more than one sub-units
+    # So this hash is ref -> [List of units]
+    comps_hash = {}
+    for c in comps:
+        cur_list = comps_hash.get(c.ref, [])
+        cur_list.append(c)
+        comps_hash[c.ref] = cur_list
     for m in GS.get_modules():
         ref = m.GetReference()
         if ref not in comps_hash:
             logger.warning(W_PCBNOSCH + '`{}` component in board, but not in schematic'.format(ref))
             continue
-        c = comps_hash[ref]
-        c.bottom = m.IsFlipped()
-        c.footprint_rot = m.GetOrientationDegrees()
-        center = GS.get_center(m)
-        c.footprint_x = center.x
-        c.footprint_y = center.y
-        (c.footprint_w, c.footprint_h) = GS.get_fp_size(m)
-        attrs = m.GetAttributes()
-        if GS.ki5():
-            # KiCad 5
-            if attrs == UI_SMD:
-                c.smd = True
-            elif attrs == UI_VIRTUAL:
-                c.virtual = True
+        for c in comps_hash[ref]:
+            c.bottom = m.IsFlipped()
+            c.footprint_rot = m.GetOrientationDegrees()
+            center = GS.get_center(m)
+            c.footprint_x = center.x
+            c.footprint_y = center.y
+            (c.footprint_w, c.footprint_h) = GS.get_fp_size(m)
+            c.has_pcb_info = True
+            attrs = m.GetAttributes()
+            if GS.ki5:
+                # KiCad 5
+                if attrs == UI_SMD:
+                    c.smd = True
+                elif attrs == UI_VIRTUAL:
+                    c.virtual = True
+                else:
+                    c.tht = True
             else:
-                c.tht = True
-        else:
-            # KiCad 6
-            if attrs & MOD_SMD:
-                c.smd = True
-            if attrs & MOD_THROUGH_HOLE:
-                c.tht = True
-            if attrs & MOD_VIRTUAL == MOD_VIRTUAL:
-                c.virtual = True
+                # KiCad 6
+                if attrs & MOD_SMD:
+                    c.smd = True
+                if attrs & MOD_THROUGH_HOLE:
+                    c.tht = True
+                if attrs & MOD_VIRTUAL == MOD_VIRTUAL:
+                    c.virtual = True
+                if attrs & MOD_EXCLUDE_FROM_POS_FILES:
+                    c.in_pos = False
+                # The PCB contains another flag for the BoM
+                # I guess it should be in sync, but: why should somebody want to unsync it?
+                if attrs & MOD_EXCLUDE_FROM_BOM:
+                    c.in_bom_pcb = False
+                if attrs & MOD_BOARD_ONLY:
+                    c.in_pcb_only = True
 
 
 def preflight_checks(skip_pre, targets):
@@ -392,6 +393,12 @@ def config_output(out, dry=False, dont_stop=False):
 def run_output(out, dont_stop=False):
     if out._done:
         return
+    if GS.global_set_text_variables_before_output and hasattr(out.options, 'variant'):
+        pre = BasePreFlight.get_preflight('set_text_variables')
+        if pre:
+            pre._variant = out.options.variant
+            pre.apply()
+            load_board()
     GS.current_output = out.name
     try:
         out.run(get_output_dir(out.dir, out))
@@ -411,7 +418,7 @@ def run_output(out, dont_stop=False):
             raise
 
 
-def generate_outputs(outputs, targets, invert, skip_pre, cli_order, no_priority, dont_stop=False):
+def _generate_outputs(outputs, targets, invert, skip_pre, cli_order, no_priority, dont_stop):
     logger.debug("Starting outputs for board {}".format(GS.pcb_file))
     # Make a list of target outputs
     n = len(targets)
@@ -470,6 +477,18 @@ def generate_outputs(outputs, targets, invert, skip_pre, cli_order, no_priority,
             run_output(out, dont_stop)
 
 
+def generate_outputs(outputs, targets, invert, skip_pre, cli_order, no_priority, dont_stop=False):
+    prj = None
+    if GS.global_restore_project:
+        # Memorize the project content to restore it at exit
+        prj = GS.read_pro()
+    try:
+        _generate_outputs(outputs, targets, invert, skip_pre, cli_order, no_priority, dont_stop)
+    finally:
+        # Restore the project file
+        GS.write_pro(prj)
+
+
 def adapt_file_name(name):
     if not name.startswith('/usr'):
         name = os.path.relpath(name)
@@ -641,7 +660,7 @@ def solve_schematic(base_dir, a_schematic=None, a_board_file=None, config=None,
                 schematic = sch
     if not schematic:
         schematics = glob(os.path.join(base_dir, '*.sch'))
-        if GS.ki6():
+        if GS.ki6:
             schematics += glob(os.path.join(base_dir, '*.kicad_sch'))
         if len(schematics) == 1:
             schematic = schematics[0]
@@ -659,7 +678,7 @@ def solve_schematic(base_dir, a_schematic=None, a_board_file=None, config=None,
                 sch = os.path.join(base_dir, config+'.sch')
                 if os.path.isfile(sch):
                     schematic = sch
-                elif GS.ki6():
+                elif GS.ki6:
                     # Try KiCad 6
                     sch = os.path.join(base_dir, config+'.kicad_sch')
                     if os.path.isfile(sch):
@@ -675,7 +694,7 @@ def solve_schematic(base_dir, a_schematic=None, a_board_file=None, config=None,
                         break
                 else:
                     # No way to select one, just take the first
-                    if GS.ki6():
+                    if GS.ki6:
                         schematic = guess_ki6_sch(schematics)
                     if not schematic:
                         schematic = schematics[0]
@@ -706,7 +725,6 @@ def solve_board_file(base_dir, a_board_file=None, sug_b=True):
         pcb = os.path.join(base_dir, os.path.splitext(schematic)[0]+'.kicad_pcb')
         if os.path.isfile(pcb):
             board_file = pcb
-            logger.info('Using PCB file: '+os.path.relpath(board_file))
     if not board_file:
         board_files = glob(os.path.join(base_dir, '*.kicad_pcb'))
         if len(board_files) == 1:
@@ -758,7 +776,7 @@ def look_for_used_layers():
     for e in list(GS.board.Zones()):
         layers.add(e.GetLayer())
     # Tracks and vias
-    via_type = 'VIA' if GS.ki5() else 'PCB_VIA'
+    via_type = 'VIA' if GS.ki5 else 'PCB_VIA'
     for e in GS.board.GetTracks():
         if e.GetClass() == via_type:
             for id in e.GetLayerSet().Seq():
@@ -789,7 +807,7 @@ def discover_files(dest_dir):
 
 
 def yaml_dump(f, tree):
-    if StrictVersion(yaml.__version__) < StrictVersion('3.14'):
+    if version_str2tuple(yaml.__version__) < (3, 14):
         f.write(yaml.dump(tree))
     else:
         # sort_keys was introduced after 3.13
@@ -956,8 +974,9 @@ def generate_examples(start_dir, dry, types):
         GS.debug_level = old_lvl
         logger.info('')
     # Try to open a browser
-    if os.environ.get('DISPLAY') and which('x-www-browser'):
-        Popen(['x-www-browser', os.path.join(GS.out_dir, 'index.html')])
+    index = os.path.join(GS.out_dir, 'index.html')
+    if os.environ.get('DISPLAY') and which('x-www-browser') and os.path.isfile(index):
+        Popen(['x-www-browser', index])
 
 
 # To avoid circular dependencies: Optionable needs it, but almost everything needs Optionable
