@@ -39,6 +39,8 @@ FAILED_EXECUTE = 25
 KICOST_ERROR = 26
 MISSING_WKS = 27
 MISSING_FILES = 28
+DIFF_TOO_BIG = 29
+NETLIST_DIFF = 30
 error_level_to_name = ['NONE',
                        'INTERNAL_ERROR',
                        'WRONG_ARGUMENTS',
@@ -67,32 +69,13 @@ error_level_to_name = ['NONE',
                        'FAILED_EXECUTE',
                        'KICOST_ERROR',
                        'MISSING_WKS',
+                       'MISSING_FILES',
+                       'DIFF_TOO_BIG',
+                       'NETLIST_DIFF',
                        ]
-CMD_EESCHEMA_DO = 'eeschema_do'
-URL_EESCHEMA_DO = 'https://github.com/INTI-CMNB/KiAuto'
-CMD_PCBNEW_RUN_DRC = 'pcbnew_do'
-URL_PCBNEW_RUN_DRC = URL_EESCHEMA_DO
-CMD_PCBNEW_PRINT_LAYERS = CMD_PCBNEW_RUN_DRC
-URL_PCBNEW_PRINT_LAYERS = URL_EESCHEMA_DO
-CMD_PCBNEW_3D = CMD_PCBNEW_RUN_DRC
-URL_PCBNEW_3D = URL_EESCHEMA_DO
-CMD_PCBNEW_GENCAD = CMD_PCBNEW_RUN_DRC
-URL_PCBNEW_GENCAD = URL_EESCHEMA_DO
-CMD_PCBNEW_IPC_NETLIST = CMD_PCBNEW_RUN_DRC
-URL_PCBNEW_IPC_NETLIST = URL_EESCHEMA_DO
-CMD_KIBOM = 'KiBOM_CLI.py'
-URL_KIBOM = 'https://github.com/INTI-CMNB/KiBoM'
-CMD_IBOM = 'generate_interactive_bom.py'
-URL_IBOM = 'https://github.com/INTI-CMNB/InteractiveHtmlBom'
-CMD_KICOST = 'kicost'
-URL_KICOST = 'https://github.com/INTI-CMNB/KiCost'
 KICOST_SUBMODULE = '../submodules/KiCost/src/kicost'
-KICAD2STEP = 'kicad2step_do'
-PCBDRAW = 'pcbdraw'
-URL_PCBDRAW = 'https://github.com/INTI-CMNB/pcbdraw'
 EXAMPLE_CFG = 'example_template.kibot.yaml'
 AUTO_SCALE = 0
-PANDOC = 'pandoc'
 KICAD_VERSION_5_99 = 5099000
 KICAD_VERSION_6_0_0 = 6000000
 KICAD_VERSION_6_0_2 = 6000002
@@ -105,6 +88,8 @@ IFILT_VAR_RENAME_KICOST = '_var_rename_kicost'
 IFILT_ROT_FOOTPRINT = '_rot_footprint'
 IFILT_KICOST_RENAME = '_kicost_rename'
 IFILT_KICOST_DNP = '_kicost_dnp'
+IFILT_EXPAND_TEXT_VARS = '_expand_text_vars'
+IFILT_DATASHEET_LINK = '_datasheet_link'
 # KiCad 5 GUI values for the attribute
 UI_THT = 0       # 1 for KiCad 6
 UI_SMD = 1       # 2 for KiCad 6
@@ -243,6 +228,16 @@ W_PDMASKFAIL = '(W089) '
 W_MISSTOOL = '(W090) '
 W_NOTYET = '(W091) '
 W_NOMATCH = '(W092) '
+W_DOWNTOOL = '(W093) '
+W_NOPREFLIGHTS = '(W094) '
+W_NOPART = '(W095) '
+W_MAXDEPTH = '(W096) '
+W_3DRESVER = '(W097) '
+W_DOWN3D = '(W098) '
+W_MISSREF = '(W099) '
+W_COPYOVER = '(W100) '
+W_PARITY = '(W101) '
+W_MISSFPINFO = '(W102) '
 # Somehow arbitrary, the colors are real, but can be different
 PCB_MAT_COLORS = {'fr1': "937042", 'fr2': "949d70", 'fr3': "adacb4", 'fr4': "332B16", 'fr5': "6cc290"}
 PCB_FINISH_COLORS = {'hal': "8b898c", 'hasl': "8b898c", 'imag': "8b898c", 'enig': "cfb96e", 'enepig': "cfb96e",
@@ -260,6 +255,10 @@ SOLDER_COLORS = {'green': ("#285e3a", "#208b47"),
 SILK_COLORS = {'black': "0b1013", 'white': "d5dce4"}
 # KiCad 6 uses IUs for SVGs, but KiCad 5 uses a very different scale based on inches
 KICAD5_SVG_SCALE = 116930/297002200
+# Some browser name to pretend
+USER_AGENT = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:102.0) Gecko/20100101 Firefox/102.0'
+# Text used to disable 3D models
+DISABLE_3D_MODEL_TEXT = '_Disabled_by_KiBot'
 
 
 class Rect(object):
@@ -299,73 +298,5 @@ def hide_stderr():
     os.dup2(newstderr, 2)
 
 
-class ToolDependencyRole(object):
-    """ Class used to define the role of a tool """
-    def __init__(self, desc=None, version=None, output=None):
-        # Is this tool mandatory
-        self.mandatory = desc is None
-        # If not mandatory, for what?
-        self.desc = desc
-        # Which version is needed?
-        self.version = version
-        # Which output needs it?
-        self.output = output
-
-
-class ToolDependency(object):
-    """ Class used to define tools needed for an output """
-    def __init__(self, output, name, url=None, url_down=None, is_python=False, deb=None, in_debian=True, extra_deb=None,
-                 roles=None, plugin_dirs=None, command=None, pypi_name=None, module_name=None, no_cmd_line_version=False,
-                 help_option=None, no_cmd_line_version_old=False):
-        # The associated output
-        self.output = output
-        # Name of the tool
-        self.name = name
-        # Name of the .deb
-        if deb is None:
-            if is_python:
-                self.deb_package = 'python3-'+name.lower()
-            else:
-                self.deb_package = name.lower()
-        else:
-            self.deb_package = deb
-        self.is_python = is_python
-        if is_python:
-            self.module_name = module_name if module_name is not None else name.lower()
-        # If this tool has an official Debian package
-        self.in_debian = in_debian
-        # Name at PyPi, can be fake for things that aren't at PyPi
-        # Is used just to indicate if a dependency will we installed from PyPi
-        self.pypi_name = pypi_name if pypi_name is not None else name
-        # Extra Debian packages needed to complement it
-        self.extra_deb = extra_deb
-        # URLs
-        self.url = url
-        self.url_down = url_down
-        # Can be installed as a KiCad plug-in?
-        self.is_kicad_plugin = plugin_dirs is not None
-        self.plugin_dirs = plugin_dirs
-        # Command we run
-        self.command = command if command is not None else name.lower()
-        self.no_cmd_line_version = no_cmd_line_version
-        self.no_cmd_line_version_old = no_cmd_line_version_old  # An old version doesn't have version
-        self.help_option = help_option if help_option is not None else '--version'
-        # Roles
-        if roles is None:
-            roles = [ToolDependencyRole()]
-        elif not isinstance(roles, list):
-            roles = [roles]
-        for r in roles:
-            r.output = output
-        self.roles = roles
-
-
-def kiauto_dependency(output, version=None):
-    role = None if version is None else ToolDependencyRole(version=version)
-    return ToolDependency(output, 'KiCad Automation tools', URL_EESCHEMA_DO, url_down=URL_EESCHEMA_DO+'/releases',
-                          in_debian=False, pypi_name='kiauto', command='pcbnew_do', roles=role)
-
-
-def git_dependency(output):
-    return ToolDependency(output, 'Git', 'https://git-scm.com/',
-                          roles=ToolDependencyRole(desc='Find commit hash and/or date'))
+def version_str2tuple(ver):
+    return tuple(map(int, ver.split('.')))
