@@ -43,7 +43,7 @@ class Optionable(object):
         self._expand_id = ''
         self._expand_ext = ''
         super().__init__()
-        for var in ['output', 'variant', 'units']:
+        for var in ['output', 'variant', 'units', 'hide_excluded']:
             glb = getattr(GS, 'global_'+var)
             if glb and hasattr(self, var):
                 setattr(self, var, glb)
@@ -212,14 +212,24 @@ class Optionable(object):
         attrs = self.get_attrs_for()
         return ((k, v) for k, v in attrs.items() if k[0] != '_')
 
+    @staticmethod
+    def _find_global_variant():
+        if GS.solved_global_variant:
+            return GS.solved_global_variant.file_id
+        return ''
+
     def _find_variant(self):
         """ Returns the text to add for the current variant.
             Also try with the globally defined variant.
             If no variant is defined an empty string is returned. """
         if hasattr(self, 'variant') and self.variant and hasattr(self.variant, 'file_id'):
             return self.variant.file_id
+        return Optionable._find_global_variant()
+
+    @staticmethod
+    def _find_global_variant_name():
         if GS.solved_global_variant:
-            return GS.solved_global_variant.file_id
+            return GS.solved_global_variant.name
         return ''
 
     def _find_variant_name(self):
@@ -228,9 +238,7 @@ class Optionable(object):
             If no variant is defined an empty string is returned. """
         if hasattr(self, 'variant') and self.variant and hasattr(self.variant, 'name'):
             return self.variant.name
-        if GS.solved_global_variant:
-            return GS.solved_global_variant.name
-        return ''
+        return Optionable._find_global_variant_name()
 
     def expand_filename_common(self, name, parent):
         """ Expansions common to the PCB and Schematic """
@@ -268,6 +276,9 @@ class Optionable(object):
             if parent and hasattr(parent, 'output_id'):
                 replace_id = _cl(parent.output_id)
             name = name.replace('%I', replace_id)
+        else:
+            name = name.replace('%v', _cl(Optionable._find_global_variant()))
+            name = name.replace('%V', _cl(Optionable._find_global_variant_name()))
         return name
 
     def expand_filename_both(self, name, is_sch=True, make_safe=True):
@@ -381,12 +392,20 @@ class BaseOptions(Optionable):
         """ Set attributes from a PCB_PLOT_PARAMS (plot options) """
         return
 
-    def expand_filename(self, dir, name, id, ext):
+    def ensure_tool(self, name):
+        """ Looks for a mandatory dependency """
+        return GS.check_tool_dep(self._parent.type, name, fatal=True)
+
+    def check_tool(self, name):
+        """ Looks for a dependency """
+        return GS.check_tool_dep(self._parent.type, name, fatal=False)
+
+    def expand_filename(self, dir, name, id, ext, make_safe=True):
         cur_id = self._expand_id
         cur_ext = self._expand_ext
         self._expand_id = id
         self._expand_ext = ext
-        name = self.expand_filename_both(name, is_sch=self._parent._sch_related)
+        name = self.expand_filename_both(name, is_sch=self._parent._sch_related, make_safe=make_safe)
         res = os.path.abspath(os.path.join(dir, name))
         self._expand_id = cur_id
         self._expand_ext = cur_ext
