@@ -50,6 +50,7 @@ class AnyLayerOptions(VariantOptions):
             """ Do not plot the component pads in the silk screen (KiCad 5.x only) """
             self.plot_sheet_reference = False
             """ *Include the frame and title block. Only available for KiCad 6 and you get a poor result
+                (i.e. always the default worksheet style, also problems expanding text variables).
                 The `pcb_print` output can do a better job for PDF, SVG, PS, EPS and PNG outputs """
             self.plot_footprint_refs = True
             """ Include the footprint references """
@@ -58,7 +59,10 @@ class AnyLayerOptions(VariantOptions):
             self.force_plot_invisible_refs_vals = False
             """ Include references and values even when they are marked as invisible """
             self.output = GS.def_global_output
-            """ *Output file name, the default KiCad name if empty """
+            """ *Output file name, the default KiCad name if empty.
+                IMPORTANT! KiCad will always create the file using its own name and then we can rename it.
+                For this reason you must avoid generating two variants at the same directory when one of
+                them uses the default KiCad name """
             self.tent_vias = True
             """ Cover the vias """
             self.uppercase_extensions = False
@@ -81,12 +85,12 @@ class AnyLayerOptions(VariantOptions):
     def _configure_plot_ctrl(self, po, output_dir):
         logger.debug("Configuring plot controller for output")
         po.SetOutputDirectory(output_dir)
-        po.SetPlotFrameRef(self.plot_sheet_reference and (not GS.ki5()))
+        po.SetPlotFrameRef(self.plot_sheet_reference and (not GS.ki5))
         po.SetPlotReference(self.plot_footprint_refs)
         po.SetPlotValue(self.plot_footprint_values)
         po.SetPlotInvisibleText(self.force_plot_invisible_refs_vals)
         po.SetExcludeEdgeLayer(self.exclude_edge_layer)
-        if GS.ki5():
+        if GS.ki5:
             po.SetPlotPadsOnSilkLayer(not self.exclude_pads_from_silkscreen)
         po.SetPlotViaOnMaskLayer(not self.tent_vias)
         # Only useful for gerber outputs
@@ -94,18 +98,6 @@ class AnyLayerOptions(VariantOptions):
         # We'll come back to this on a per-layer basis
         po.SetSkipPlotNPTH_Pads(False)
 
-    def filter_components(self, board):
-        """  Apply the variants and filters """
-        if not self._comps:
-            return None
-        self.comps_hash = self.get_refs_hash()
-        self.cross_modules(board, self.comps_hash)
-        return self.remove_paste_and_glue(board, self.comps_hash)
-
-    def unfilter_components(self, board):
-        self.uncross_modules(board, self.comps_hash)
-        self.restore_paste_and_glue(board, self.comps_hash)
-
     def compute_name(self, k_filename, output_dir, output, id, suffix):
         if output:
             filename = self.expand_filename(output_dir, output, suffix, os.path.splitext(k_filename)[1][1:])
@@ -136,7 +128,7 @@ class AnyLayerOptions(VariantOptions):
             jobfile_writer = GERBER_JOBFILE_WRITER(GS.board)
         plot_ctrl.SetColorMode(True)
         # Apply the variants and filters
-        exclude = self.filter_components(GS.board)
+        exclude = self.filter_pcb_components(GS.board)
         # Plot every layer in the output
         generated = {}
         layers = Layer.solve(layers)
@@ -164,7 +156,7 @@ class AnyLayerOptions(VariantOptions):
             logger.debug("Plotting layer `{}` to `{}`".format(la, filename))
             plot_ctrl.PlotLayer()
             plot_ctrl.ClosePlot()
-            if self.output:
+            if self.output and k_filename != filename:
                 os.rename(k_filename, filename)
             if create_job:
                 jobfile_writer.AddGbrFile(id, os.path.basename(filename))
@@ -193,7 +185,7 @@ class AnyLayerOptions(VariantOptions):
                 f.write(content)
         # Restore the eliminated layers
         if exclude:
-            self.unfilter_components(GS.board)
+            self.unfilter_pcb_components(GS.board)
 
     def solve_extension(self, layer):
         if self._plot_format == PLOT_FORMAT_GERBER and self.use_protel_extensions:
@@ -231,7 +223,7 @@ class AnyLayerOptions(VariantOptions):
         self.force_plot_invisible_refs_vals = po.GetPlotInvisibleText()
         # viasonmask
         self.tent_vias = not po.GetPlotViaOnMaskLayer()
-        if GS.ki5():
+        if GS.ki5:
             # padsonsilk
             self.exclude_pads_from_silkscreen = not po.GetPlotPadsOnSilkLayer()
 
