@@ -7,10 +7,9 @@ import os
 from shutil import rmtree
 from .pre_base import BasePreFlight
 from .gs import GS
-from .kiplot import check_script, exec_with_retry, add_extra_options
-from .misc import CMD_PCBNEW_PRINT_LAYERS, URL_PCBNEW_PRINT_LAYERS, PDF_PCB_PRINT, kiauto_dependency
+from .kiplot import exec_with_retry, add_extra_options
+from .misc import PDF_PCB_PRINT
 from .out_base import VariantOptions
-from .registrable import RegDependency
 from .macros import macros, document, output_class  # noqa: F401
 from .drill_marks import add_drill_marks, DRILL_MARKS_MAP
 from .layer import Layer
@@ -19,10 +18,6 @@ from . import log
 logger = log.get_logger()
 
 
-def register_deps(pre):
-    RegDependency.register(kiauto_dependency(pre+'_pcb_print', (1, 6, 7)))
-
-
 class Any_PCB_PrintOptions(VariantOptions):
     def __init__(self):
         with document:
@@ -39,7 +34,8 @@ class Any_PCB_PrintOptions(VariantOptions):
             self.mirror = False
             """ Print mirrored (X axis inverted). ONLY for KiCad 6 """
             self.hide_excluded = False
-            """ Hide components in the Fab layer that are marked as excluded by a variant """
+            """ Hide components in the Fab layer that are marked as excluded by a variant.
+                Affected by global options """
             self.title = ''
             """ Text used to replace the sheet title. %VALUE expansions are allowed.
                 If it starts with `+` the text is concatenated """
@@ -58,20 +54,13 @@ class Any_PCB_PrintOptions(VariantOptions):
         super().config(parent)
         self.drill_marks = DRILL_MARKS_MAP[self.drill_marks]
 
-    def filter_components(self, board, force_copy):
+    def filter_components(self, force_copy):
         if not self._comps and not force_copy:
             return GS.pcb_file, None
-        comps_hash = self.get_refs_hash()
-        self.cross_modules(board, comps_hash)
-        self.remove_paste_and_glue(board, comps_hash)
-        if self.hide_excluded:
-            self.remove_fab(board, comps_hash)
+        self.filter_pcb_components(GS.board)
         # Save the PCB to a temporal dir
         fname, pcb_dir = self.save_tmp_dir_board('pdf_pcb_print')
-        self.uncross_modules(board, comps_hash)
-        self.restore_paste_and_glue(board, comps_hash)
-        if self.hide_excluded:
-            self.restore_fab(board, comps_hash)
+        self.unfilter_pcb_components(GS.board)
         return fname, pcb_dir
 
     def get_targets(self, out_dir):
@@ -79,9 +68,9 @@ class Any_PCB_PrintOptions(VariantOptions):
 
     def run(self, output, svg=False):
         super().run(self._layers)
-        check_script(CMD_PCBNEW_PRINT_LAYERS, URL_PCBNEW_PRINT_LAYERS, '1.6.7')
+        command = self.ensure_tool('KiAuto')
         # Output file name
-        cmd = [CMD_PCBNEW_PRINT_LAYERS, 'export', '--output_name', output]
+        cmd = [command, 'export', '--output_name', output]
         if BasePreFlight.get_option('check_zone_fills'):
             cmd.append('-f')
         cmd.extend(['--scaling', str(self.scaling), '--pads', str(self.drill_marks)])
@@ -98,12 +87,12 @@ class Any_PCB_PrintOptions(VariantOptions):
         if svg:
             cmd.append('--svg')
         self.set_title(self.title)
-        board_name, board_dir = self.filter_components(GS.board, self.title != '')
+        board_name, board_dir = self.filter_components(self.title != '')
         cmd.extend([board_name, os.path.dirname(output)])
         cmd, video_remove = add_extra_options(cmd)
         # Add the layers
         cmd.extend([la.layer for la in self._layers])
-        if GS.ki6() and self.force_edge_cuts and not self.separated:
+        if GS.ki6 and self.force_edge_cuts and not self.separated:
             cmd.append('Edge.Cuts')
         # Execute it
         ret = exec_with_retry(cmd)
@@ -113,7 +102,7 @@ class Any_PCB_PrintOptions(VariantOptions):
             logger.debug('Removing temporal variant dir `{}`'.format(board_dir))
             rmtree(board_dir)
         if ret:
-            logger.error(CMD_PCBNEW_PRINT_LAYERS+' returned %d', ret)
+            logger.error(command+' returned %d', ret)
             exit(PDF_PCB_PRINT)
         if video_remove:
             video_name = os.path.join(self.expand_filename_pcb(GS.out_dir), 'pcbnew_export_screencast.ogv')
