@@ -4,24 +4,35 @@
 # License: MIT
 # Project: KiBot (formerly KiPlot)
 """
-Internal BoM (Bill of Materials) output for KiBot.
-This is somehow compatible with KiBoM.
+# Internal BoM (Bill of Materials) output for KiBot.
+# This is somehow compatible with KiBoM.
+Dependencies:
+  - from: KiCost
+    role: Find components costs and specs
+    version: 1.1.8
+  - name: XLSXWriter
+    role: Create XLSX files
+    python_module: true
+    debian: python3-xlsxwriter
+    arch: python-xlsxwriter
+    downloader: python
 """
+import csv
+from copy import deepcopy
 import os
 import re
-from copy import deepcopy
 from .gs import GS
-from .misc import W_BADFIELD, W_NEEDSPCB, DISTRIBUTORS, ToolDependency, ToolDependencyRole, URL_KICOST
+from .misc import W_BADFIELD, W_NEEDSPCB, DISTRIBUTORS, IFILT_EXPAND_TEXT_VARS, W_NOPART, W_MISSREF
 from .optionable import Optionable, BaseOptions
-from .registrable import RegOutput, RegDependency
+from .registrable import RegOutput
 from .error import KiPlotConfigurationError
 from .kiplot import get_board_comps_data, load_any_sch
+from .kicad.v5_sch import SchematicComponent, SchematicField
 from .bom.columnlist import ColumnList, BoMError
 from .bom.bom import do_bom
-from .bom.xlsx_writer import KICOST_SUPPORT
 from .var_kibom import KiBoM
 from .fil_base import (BaseFilter, apply_exclude_filter, apply_fitted_filter, apply_fixed_filter, reset_filters,
-                       KICOST_NAME_TRANSLATIONS)
+                       KICOST_NAME_TRANSLATIONS, apply_pre_transform)
 from .macros import macros, document, output_class  # noqa: F401
 from . import log
 # To debug the `with document` we can use:
@@ -37,10 +48,20 @@ DEFAULT_ALIASES = [['r', 'r_small', 'res', 'resistor'],
                    ['zener', 'zenersmall'],
                    ['d', 'diode', 'd_small'],
                    ]
-RegDependency.register(ToolDependency('bom', 'KiCost', URL_KICOST, url_down=URL_KICOST+'/releases', in_debian=False,
-                                      roles=ToolDependencyRole(desc='Find components costs and specs', version=(1, 1, 8))))
-RegDependency.register(ToolDependency('bom', 'XLSXWriter', is_python=True,
-                                      roles=ToolDependencyRole(desc='Create XLSX files')))
+
+
+class CompsFromCSV(object):
+    """ Class used to fake an schematic using a CSV file """
+    def __init__(self, fname, comps):
+        super().__init__()
+        self.revision = ''
+        self.date = GS.format_date('', fname, 'SCH')
+        self.title = os.path.basename(fname)
+        self.company = ''
+        self.comps = comps
+
+    def get_components(self):
+        return self.comps
 
 
 class BoMJoinField(Optionable):
@@ -357,6 +378,8 @@ class Aggregate(Optionable):
             """ A prefix to add to all the references from this project """
             self.number = 1
             """ Number of boards to build (components multiplier). Use negative to subtract """
+            self.delimiter = ','
+            """ Delimiter used for CSV files """
 
     def config(self, parent):
         super().config(parent)
@@ -383,7 +406,7 @@ class BoMOptions(BaseOptions):
             self.ignore_dnf = True
             """ *Exclude DNF (Do Not Fit) components """
             self.fit_field = 'Config'
-            """ Field name used for internal filters """
+            """ Field name used for internal filters (not for variants) """
             self.use_alt = False
             """ Print grouped references in the alternate compressed style eg: R1-R7,R18 """
             self.columns = BoMColumns
@@ -405,15 +428,21 @@ class BoMOptions(BaseOptions):
             self.csv = BoMCSV
             """ *[dict] Options for the CSV, TXT and TSV formats """
             # * Filters
+            self.pre_transform = Optionable
+            """ [string|list(string)='_none'] Name of the filter to transform fields before applying other filters.
+                This option is for simple cases, consider using a full variant for complex cases """
             self.exclude_filter = Optionable
             """ [string|list(string)='_mechanical'] Name of the filter to exclude components from BoM processing.
-                The default filter excludes test points, fiducial marks, mounting holes, etc """
+                The default filter excludes test points, fiducial marks, mounting holes, etc.
+                This option is for simple cases, consider using a full variant for complex cases """
             self.dnf_filter = Optionable
             """ [string|list(string)='_kibom_dnf'] Name of the filter to mark components as 'Do Not Fit'.
-                The default filter marks components with a DNF value or DNF in the Config field """
+                The default filter marks components with a DNF value or DNF in the Config field.
+                This option is for simple cases, consider using a full variant for complex cases """
             self.dnc_filter = Optionable
             """ [string|list(string)='_kibom_dnc'] Name of the filter to mark components as 'Do Not Change'.
-                The default filter marks components with a DNC value or DNC in the Config field """
+                The default filter marks components with a DNC value or DNC in the Config field.
+                This option is for simple cases, consider using a full variant for complex cases """
             # * Grouping criteria
             self.group_connectors = True
             """ Connectors with the same footprints will be grouped together, independent of the name of the connector """
@@ -447,7 +476,12 @@ class BoMOptions(BaseOptions):
                 By default the field indicated in `fit_field`, the field used for variants and
                 the field `part` are excluded """
             self.aggregate = Aggregate
-            """ [list(dict)] Add components from other projects """
+            """ [list(dict)] Add components from other projects.
+                You can use CSV files, the first row must contain the names of the fields.
+                The `Reference` and `Value` are mandatory, in most cases `Part` is also needed.
+                The `Part` column should contain the name/type of the component. This is important for
+                passive components (R, L, C, etc.). If this information isn't available consider
+                configuring the grouping to exclude the `Part`. """
             self.ref_id = ''
             """ A prefix to add to all the references from this project. Used for multiple projects """
             self.source_by_id = False
@@ -475,6 +509,17 @@ class BoMOptions(BaseOptions):
             """ [string|list(string)='no,yes'] Values for the `Footprint Populate` column """
             self.footprint_type_values = Optionable
             """ [string|list(string)='SMD,THT,VIRTUAL'] Values for the `Footprint Type` column """
+            self.expand_text_vars = True
+            """ Expand KiCad 6 text variables after applying all filters and variants.
+                This is done using a **_expand_text_vars** filter.
+                If you need to customize the filter, or apply it before, you can disable this option and
+                add a custom filter to the filter chain """
+            self.exclude_marked_in_sch = True
+            """ Exclude components marked with *Exclude from bill of materials* in the schematic.
+                This is a KiCad 6 option """
+            self.exclude_marked_in_pcb = False
+            """ Exclude components marked with *Exclude from BOM* in the PCB.
+                This is a KiCad 6 option """
         self._format_example = 'CSV'
         self._footprint_populate_values_example = 'no,yes'
         self._footprint_type_values_example = 'SMD,THT,VIRTUAL'
@@ -515,8 +560,8 @@ class BoMOptions(BaseOptions):
             self.variant.variant = []
             self.variant.name = 'default'
             # Delegate any filter to the variant
-            self.variant.set_def_filters(self.exclude_filter, self.dnf_filter, self.dnc_filter)
-            self.exclude_filter = self.dnf_filter = self.dnc_filter = None
+            self.variant.set_def_filters(self.exclude_filter, self.dnf_filter, self.dnc_filter, self.pre_transform)
+            self.exclude_filter = self.dnf_filter = self.dnc_filter = self.pre_transform = None
             self.variant.config(self)  # Fill or adjust any detail
 
     def process_columns_config(self, cols, valid_columns, extra_columns, add_all=True):
@@ -626,6 +671,7 @@ class BoMOptions(BaseOptions):
         if isinstance(self.component_aliases, type):
             self.component_aliases = DEFAULT_ALIASES
         # Filters
+        self.pre_transform = BaseFilter.solve_filter(self.pre_transform, 'pre_transform')
         self.exclude_filter = BaseFilter.solve_filter(self.exclude_filter, 'exclude_filter')
         self.dnf_filter = BaseFilter.solve_filter(self.dnf_filter, 'dnf_filter')
         self.dnc_filter = BaseFilter.solve_filter(self.dnc_filter, 'dnc_filter')
@@ -671,6 +717,91 @@ class BoMOptions(BaseOptions):
         (self.columns_ce, self.column_levels_ce, self.column_comments_ce, self.column_rename_ce,
          self.join_ce) = self.process_columns_config(self.cost_extra_columns, valid_columns, extra_columns, add_all=False)
 
+    def get_ref_index(self, header, fname):
+        ref_n = ColumnList.COL_REFERENCE_L
+        ref_index = None
+        try:
+            ref_index = header.index(ref_n)
+        except ValueError:
+            try:
+                ref_index = header.index(ref_n[:-1])
+            except ValueError:
+                msg = 'Missing `{}` in aggregated file `{}`'.format(ref_n, fname)
+                if GS.global_csv_accept_no_ref:
+                    logger.warning(W_MISSREF+msg)
+                else:
+                    raise KiPlotConfigurationError(msg)
+        return ref_index
+
+    def load_csv(self, fname, project, delimiter):
+        """ Load components from a CSV file """
+        comps = []
+        logger.debug('Importing components from `{}`'.format(fname))
+        with open(fname) as csvfile:
+            reader = csv.reader(csvfile, delimiter=delimiter)
+            header = [x.lower() for x in next(reader)]
+            logger.debugl(1, '- CSV header {}'.format(header))
+            # The header must contain at least the reference and the value
+            ref_index = self.get_ref_index(header, fname)
+            try:
+                val_index = header.index(ColumnList.COL_VALUE_L)
+            except ValueError:
+                raise KiPlotConfigurationError('Missing `{}` in aggregated file `{}`'.format(ColumnList.COL_VALUE_L, fname))
+            # Optional important fields:
+            fp_index = None
+            try:
+                fp_index = header.index(ColumnList.COL_FP_L)
+            except ValueError:
+                pass
+            ds_index = None
+            try:
+                ds_index = header.index(ColumnList.COL_DATASHEET_L)
+            except ValueError:
+                pass
+            pn_index = None
+            try:
+                pn_index = header.index(ColumnList.COL_PART_L)
+            except ValueError:
+                logger.warning(W_NOPART+'No `Part` specified, using `Value` instead, this can impact the grouping')
+            min_num = len(header)
+            for r in reader:
+                c = SchematicComponent()
+                c.unit = 0
+                c.project = project
+                c.lib = ''
+                c.ref = c.f_ref = c.ref_prefix = ''
+                c.ref_suffix = '?'
+                c.sheet_path_h = '/'+project
+                for n, f in enumerate(r):
+                    number = None
+                    if n == ref_index:
+                        c.ref = c.f_ref = str(f)
+                        c.split_ref()
+                        number = 0
+                    elif n == val_index:
+                        c.value = str(f)
+                        if pn_index is None:
+                            c.name = str(f)
+                        number = 1
+                    elif n == fp_index:
+                        c.footprint = str(f)
+                        number = 2
+                    elif ds_index:
+                        c.datasheet = str(f)
+                        number = 3
+                    elif n == pn_index:
+                        c.name = str(f)
+                        number = -1
+                    fld = SchematicField()
+                    fld.number = min_num+n if number is None else number
+                    fld.value = str(f)
+                    fld.name = header[n]
+                    c.add_field(fld)
+                comps.append(c)
+                logger.debugl(2, '- Adding component {}'.format(c))
+        comps.sort(key=lambda g: g.ref)
+        return CompsFromCSV(fname, comps)
+
     def aggregate_comps(self, comps):
         self.qtys = {GS.sch_basename: self.number}
         for prj in self.aggregate:
@@ -679,7 +810,11 @@ class BoMOptions(BaseOptions):
             logger.debug('Adding components from project {} ({}) using reference id `{}`'.
                          format(prj.name, prj.file, prj.ref_id))
             self.qtys[prj.name] = prj.number
-            prj.sch = load_any_sch(prj.file, prj.name)
+            ext = os.path.splitext(prj.file)[1]
+            if ext == '.sch' or ext == '.kicad_sch':
+                prj.sch = load_any_sch(prj.file, prj.name)
+            else:
+                prj.sch = self.load_csv(prj.file, prj.name, prj.delimiter)
             new_comps = prj.sch.get_components()
             for c in new_comps:
                 c.ref = prj.ref_id+c.ref
@@ -689,6 +824,10 @@ class BoMOptions(BaseOptions):
 
     def run(self, output):
         format = self.format.lower()
+        if format == 'xlsx':
+            if self.xlsx.kicost:
+                self.ensure_tool('KiCost')
+            self.ensure_tool('XLSXWriter')
         # Add some info needed for the output to the config object.
         # So all the configuration is contained in one object.
         self.source = GS.sch_basename
@@ -711,11 +850,25 @@ class BoMOptions(BaseOptions):
         self.aggregate_comps(comps)
         # Apply all the filters
         reset_filters(comps)
+        if self.exclude_marked_in_sch:
+            for c in comps:
+                if c.included:
+                    c.included = c.in_bom
+        if self.exclude_marked_in_pcb:
+            for c in comps:
+                if c.included:
+                    c.included = c.in_bom_pcb
+        comps = apply_pre_transform(comps, self.pre_transform)
         apply_exclude_filter(comps, self.exclude_filter)
         apply_fitted_filter(comps, self.dnf_filter)
         apply_fixed_filter(comps, self.dnc_filter)
         # Apply the variant
         comps = self.variant.filter(comps)
+        # Now expand the text variables, the user can disable it and insert a customized filter
+        # in the variant or even before.
+        if self.expand_text_vars:
+            comps = apply_pre_transform(comps, BaseFilter.solve_filter(IFILT_EXPAND_TEXT_VARS, 'KiCad 6 text vars',
+                                                                       is_transform=True))
         # We add the main project to the aggregate list so do_bom sees a complete list
         base_sch = Aggregate()
         base_sch.file = GS.sch_file
@@ -854,17 +1007,22 @@ class BoM(BaseOutput):  # noqa: F821
         if join_fields:
             logger.debug(' - Fields to join with Value: {}'.format(join_fields))
         # Create a generic version
-        for fmt in ['HTML', 'CSV', 'TXT', 'TSV', 'XML', 'XLSX']:
+        SIMP_FMT = ['HTML', 'CSV', 'TXT', 'TSV', 'XML']
+        XYRS_FMT = ['HTML']
+        if GS.check_tool(name, 'XLSXWriter') is not None:
+            SIMP_FMT.append('XLSX')
+            XYRS_FMT.append('XLSX')
+        for fmt in SIMP_FMT:
             outs.append(BoM.create_bom(fmt, 'Generic', group_fields, join_fields, fld_names))
         if GS.board:
             # Create an example showing the positional fields
             cols = ColumnList.COLUMNS_DEFAULT + ColumnList.COLUMNS_EXTRA
-            for fmt in ['HTML', 'XLSX']:
+            for fmt in XYRS_FMT:
                 gb = BoM.create_bom(fmt, 'Positional', group_fields, None, fld_names, cols)
                 gb['options'][fmt.lower()] = {'style': 'modern-red'}
                 outs.append(gb)
         # Create a costs version
-        if KICOST_SUPPORT:  # and dists?
+        if GS.check_tool(name, 'KiCost') is not None:  # and dists?
             logger.debug(' - KiCost distributors {}'.format(dists))
             grp = group_fields
             if group_fields:
