@@ -9,12 +9,11 @@ import requests
 from .out_base import VariantOptions
 from .fil_base import DummyFilter
 from .error import KiPlotConfigurationError
-from .misc import W_UNKFLD, W_ALRDOWN, W_FAILDL
+from .misc import W_UNKFLD, W_ALRDOWN, W_FAILDL, USER_AGENT
 from .gs import GS
 from .macros import macros, document, output_class  # noqa: F401
 from . import log
 logger = log.get_logger()
-USER_AGENT = 'Mozilla/5.0 (Windows NT 5.2; rv:2.0.1) Gecko/20100101 Firefox/4.0.1'
 
 
 def is_url(ds):
@@ -41,6 +40,7 @@ class Download_Datasheets_Options(VariantOptions):
             """ Instead of download things we already downloaded use symlinks """
         # Used to collect the targets
         self._dry = False
+        self._unkown_is_error = True
 
     def config(self, parent):
         super().config(parent)
@@ -69,6 +69,12 @@ class Download_Datasheets_Options(VariantOptions):
                 except requests.exceptions.ReadTimeout:
                     logger.warning(W_FAILDL+'Timeout during download `{}`'.format(ds))
                     return None
+                except requests.exceptions.ConnectionError:
+                    logger.warning(W_FAILDL+'Connection error during download `{}`'.format(ds))
+                    return None
+                except requests.exceptions.SSLError:
+                    logger.warning(W_FAILDL+'SSL Error during download `{}`'.format(ds))
+                    return None
                 if r.status_code != 200:
                     logger.warning(W_FAILDL+'Failed to download `{}`'.format(ds))
                     return None
