@@ -8,7 +8,7 @@ from .out_any_layer import AnyLayer
 from .drill_marks import DrillMarks
 from .gs import GS
 from .macros import macros, document, output_class  # noqa: F401
-if GS.ki6():
+if GS.ki6:
     from pcbnew import DXF_UNITS_MILLIMETERS, DXF_UNITS_INCHES
 else:
     DXF_UNITS_MILLIMETERS = 1
@@ -22,7 +22,8 @@ class DXFOptions(DrillMarks):
             self.use_aux_axis_as_origin = False
             """ Use the auxiliary axis as origin for coordinates """
             self.polygon_mode = True
-            """ Plot using the contour, instead of the center line """
+            """ Plot using the contour, instead of the center line.
+                You must disable it to get the dimensions (See https://gitlab.com/kicad/code/kicad/-/issues/11901) """
             self.metric_units = False
             """ Use mm instead of inches """
             self.sketch_plot = False
