@@ -3,30 +3,34 @@
 # Copyright (c) 2020-2022 Instituto Nacional de Tecnología Industrial
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
+"""
+Dependencies:
+  - name: Interactive HTML BoM
+    role: mandatory
+    github: INTI-CMNB/InteractiveHtmlBom
+    command: generate_interactive_bom.py
+    no_cmd_line_version_old: true
+    plugin_dirs:
+      - InteractiveHtmlBom
+      - InteractiveHtmlBom/InteractiveHtmlBom
+      - org_openscopeproject_InteractiveHtmlBom
+      - org_openscopeproject_InteractiveHtmlBom/InteractiveHtmlBom
+    version: 2.4.1.4
+    downloader: pytool
+    id: ibom
+"""
 import os
 from subprocess import (check_output, STDOUT, CalledProcessError)
-from shutil import which
-from .misc import (CMD_IBOM, URL_IBOM, BOM_ERROR, W_EXTNAME, ToolDependency, ToolDependencyRole, W_NONETLIST)
-from .gs import (GS)
-from .kiplot import check_script, search_as_plugin
+from shutil import which, rmtree
+from tempfile import mkdtemp
+from .misc import BOM_ERROR, W_EXTNAME, W_NONETLIST
+from .gs import GS
 from .out_base import VariantOptions
-from .registrable import RegDependency
 from .macros import macros, document, output_class  # noqa: F401
 from . import log
 
 logger = log.get_logger()
 WARNING_MIX = "Avoid using it in conjunction with IBoM native filtering options"
-PLUGIN_NAMES = ['InteractiveHtmlBom', 'InteractiveHtmlBom/InteractiveHtmlBom',
-                'org_openscopeproject_InteractiveHtmlBom/InteractiveHtmlBom']
-RegDependency.register(ToolDependency('ibom', 'Interactive HTML BoM', URL_IBOM, url_down=URL_IBOM+'/releases',
-                                      command=CMD_IBOM, in_debian=False, no_cmd_line_version_old=True,
-                                      plugin_dirs=PLUGIN_NAMES, roles=ToolDependencyRole(version=(2, 4, 1, 4))))
-
-
-def check_tool():
-    tool = search_as_plugin(CMD_IBOM, PLUGIN_NAMES)
-    check_script(tool, URL_IBOM)
-    return tool
 
 
 class IBoMOptions(VariantOptions):
@@ -149,7 +153,7 @@ class IBoMOptions(VariantOptions):
 
     def run(self, name):
         super().run(name)
-        tool = check_tool()
+        tool = self.ensure_tool('ibom')
         logger.debug('Doing Interactive BoM')
         # Tell ibom we don't want to use the screen
         os.environ['INTERACTIVE_HTML_BOM_NO_DISPLAY'] = ''
@@ -162,19 +166,35 @@ class IBoMOptions(VariantOptions):
             cur = os.path.join(output_dir, 'ibom.html')
         else:
             output_dir = name
-        cmd = [tool, GS.pcb_file, '--dest-dir', output_dir, '--no-browser', ]
+        # Solve the variants stuff
+        ori_extra_data_file = self.extra_data_file
+        net_dir = None
+        pcb_name = GS.pcb_file
+        if self._comps:
+            # Write a custom netlist to a temporal dir
+            net_dir = mkdtemp(prefix='tmp-kibot-ibom-')
+            netlist = os.path.join(net_dir, GS.pcb_basename+'.xml')
+            self.extra_data_file = netlist
+            logger.debug('Creating variant netlist `{}`'.format(netlist))
+            with open(netlist, 'wb') as f:
+                GS.sch.save_netlist(f, self._comps)
+            # Write a board with the filtered values applied
+            self.sch_fields_to_pcb(self._comps, GS.board)
+            pcb_name, _ = self.save_tmp_dir_board('ibom', force_dir=net_dir)
+        else:
+            # Check if the user wants extra_fields but there is no data about them (#68)
+            if self.need_extra_fields() and not os.path.isfile(self.extra_data_file):
+                logger.warning(W_NONETLIST+'iBoM needs information about user defined fields and no netlist provided')
+                if self._extra_data_file_guess:
+                    logger.warning(W_NONETLIST+'Create a BoM in XML format or use the `update_xml` preflight')
+                    # If the name of the netlist is just a guess remove it from the options
+                    self.extra_data_file = ''
+                else:
+                    logger.warning(W_NONETLIST+"The file name used in `extra_data_file` doesn't exist")
+        cmd = [tool, pcb_name, '--dest-dir', output_dir, '--no-browser', ]
         if not which(tool) and not os.access(tool, os.X_OK):
             # Plugin could be installed without execute flags
             cmd.insert(0, 'python3')
-        # Check if the user wants extra_fields but there is no data about them (#68)
-        if self.need_extra_fields() and not os.path.isfile(self.extra_data_file):
-            logger.warning(W_NONETLIST+'iBoM needs information about user defined fields and no netlist provided')
-            if self._extra_data_file_guess:
-                logger.warning(W_NONETLIST+'Create a BoM in XML format or use the `update_xml` preflight')
-                # If the name of the netlist is just a guess remove it from the options
-                self.extra_data_file = ''
-            else:
-                logger.warning(W_NONETLIST+"The file name used in `extra_data_file` doesn't exist")
         # Apply variants/filters
         to_remove = ','.join(self.get_not_fitted_refs())
         if self.blacklist and to_remove:
@@ -182,7 +202,7 @@ class IBoMOptions(VariantOptions):
         self.blacklist += to_remove
         # Convert attributes into options
         for k, v in self.get_attrs_gen():
-            if not v or k in ['output', 'variant', 'dnf_filter']:
+            if not v or k in ['output', 'variant', 'dnf_filter', 'pre_transform']:
                 continue
             cmd.append(BaseOutput.attr2longopt(k))  # noqa: F821
             if not isinstance(v, bool):  # must be str/(int, float)
@@ -202,6 +222,14 @@ class IBoMOptions(VariantOptions):
                 if "'PCB_SHAPE' object has no attribute 'GetAngle'" in e.output.decode():
                     logger.error("Update Interactive HTML BoM your version doesn't support KiCad 6 files")
             exit(BOM_ERROR)
+        finally:
+            if net_dir:
+                logger.debug('Removing temporal variant dir `{}`'.format(net_dir))
+                rmtree(net_dir)
+                # Restore the PCB properties and values
+                self.restore_sch_fields_to_pcb(GS.board)
+            # Restore the real name selected
+            self.extra_data_file = ori_extra_data_file
         logger.debug('Output from command:\n'+cmd_output_dec+'\n')
         if output:
             logger.debug('Renaming output file: {} -> {}'.format(cur, output))
@@ -219,17 +247,14 @@ class IBoM(BaseOutput):  # noqa: F821
         with document:
             self.options = IBoMOptions
             """ *[dict] Options for the `ibom` output """
+        self._category = ['Schematic/BoM', 'PCB/fabrication/assembly']
 
     def get_dependencies(self):
         return self.options.get_dependencies()
 
     @staticmethod
     def get_conf_examples(name, layers, templates):
-        enabled = True
-        try:
-            check_tool()
-        except SystemExit:
-            enabled = False
-        if not enabled:
+        tool = GS.check_tool(name, 'ibom')
+        if tool is None:
             return None
         return BaseOutput.simple_conf_examples(name, 'Interactive HTML BoM', 'Assembly')  # noqa: F821
