@@ -3,24 +3,29 @@
 # Copyright (c) 2020-2022 Instituto Nacional de Tecnología Industrial
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
+"""
+Dependencies:
+  - name: KiBoM
+    role: mandatory
+    github: INTI-CMNB/KiBoM
+    command: KiBOM_CLI.py
+    version: 1.8.0
+    downloader: pytool
+"""
 import os
 from re import search
 from tempfile import NamedTemporaryFile
 from subprocess import (check_output, STDOUT, CalledProcessError)
-from .misc import (CMD_KIBOM, URL_KIBOM, BOM_ERROR, ToolDependency, ToolDependencyRole, W_EXTNAME)
-from .kiplot import (check_script)
-from .gs import (GS)
+from .misc import BOM_ERROR, W_EXTNAME
+from .gs import GS
 from .optionable import Optionable, BaseOptions
 from .error import KiPlotConfigurationError
 from .bom.columnlist import ColumnList
-from .registrable import RegDependency
 from .macros import macros, document, output_class  # noqa: F401
 from . import log
 
 logger = log.get_logger()
 CONFIG_FILENAME = 'config.kibom.ini'
-RegDependency.register(ToolDependency('kibom', 'KiBoM', URL_KIBOM, url_down=URL_KIBOM+'/releases',
-                                      command=CMD_KIBOM, in_debian=False, roles=ToolDependencyRole(version=(1, 8, 0))))
 
 
 class KiBoMRegex(Optionable):
@@ -183,7 +188,7 @@ class KiBoMConfig(Optionable):
         """ Create a list of valid columns """
         if not GS.sch:
             return ColumnList.COLUMNS_DEFAULT
-        check_script(CMD_KIBOM, URL_KIBOM, '1.8.0')
+        command = GS.ensure_tool('kibom', 'KiBoM')
         config = None
         csv = None
         columns = None
@@ -192,7 +197,7 @@ class KiBoMConfig(Optionable):
             config = os.path.abspath(KiBoMConfig._create_minimal_ini())
             with NamedTemporaryFile(mode='w', suffix='.csv', delete=False) as f:
                 csv = f.name
-            cmd = [CMD_KIBOM, '--cfg', config, '-d', os.path.dirname(csv), '-s', ',', xml, csv]
+            cmd = [command, '--cfg', config, '-d', os.path.dirname(csv), '-s', ',', xml, csv]
             logger.debug('Running: '+str(cmd))
             cmd_output = check_output(cmd, stderr=STDOUT)
             with open(csv, 'rt') as f:
@@ -337,7 +342,7 @@ class KiBoMOptions(BaseOptions):
                 variants with the ';' (semicolon) character.
                 This isn't related to the KiBot concept of variants """
             self.conf = KiBoMConfig
-            """ [string|dict] BoM configuration file, relative to PCB.
+            """ [string|dict] BoM configuration file, relative to PCB. Environment variables and ~ allowed.
                 You can also define the configuration here, will be stored in `config.kibom.ini` """
             self.separator = ','
             """ CSV Separator """
@@ -370,10 +375,12 @@ class KiBoMOptions(BaseOptions):
         return []
 
     def run(self, name):
-        check_script(CMD_KIBOM, URL_KIBOM, '1.8.0')
+        kibom_command = self.ensure_tool('KiBoM')
         format = self.format.lower()
         prj = GS.sch_no_ext
-        config = os.path.join(GS.sch_dir, self.conf)
+        config = os.path.expandvars(os.path.expanduser(self.conf))
+        if not os.path.isabs(config):
+            config = os.path.join(GS.sch_dir, config)
         if self.output:
             force_output = True
             output = name
@@ -383,7 +390,7 @@ class KiBoMOptions(BaseOptions):
             output = os.path.basename(prj)+'.'+format
             output_dir = name
         logger.debug('Doing BoM, format {} prj: {} config: {} output: {}'.format(format, prj, config, output))
-        cmd = [CMD_KIBOM,
+        cmd = [kibom_command,
                '-n', str(self.number),
                '--cfg', config,
                '-s', self.separator,
