@@ -3,18 +3,22 @@
 # Copyright (c) 2021-2022 Instituto Nacional de Tecnología Industrial
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
+"""
+Dependencies:
+  - from: KiCost
+    role: mandatory
+    version: 1.1.7
+"""
 import os
 from os.path import isfile, abspath, join, dirname
 from subprocess import check_output, STDOUT, CalledProcessError
 from tempfile import mkdtemp
 from shutil import rmtree
-from .misc import (CMD_KICOST, URL_KICOST, BOM_ERROR, DISTRIBUTORS, W_UNKDIST, ISO_CURRENCIES, W_UNKCUR, KICOST_SUBMODULE,
-                   W_KICOSTFLD, W_MIXVARIANT, ToolDependency, ToolDependencyRole)
-from .registrable import RegDependency
+from .misc import (BOM_ERROR, DISTRIBUTORS, W_UNKDIST, ISO_CURRENCIES, W_UNKCUR, KICOST_SUBMODULE,
+                   W_KICOSTFLD, W_MIXVARIANT)
 from .error import KiPlotConfigurationError
 from .optionable import Optionable
 from .gs import GS
-from .kiplot import check_script
 from .out_base import VariantOptions
 from .macros import macros, document, output_class  # noqa: F401
 from .fil_base import FieldRename
@@ -22,8 +26,6 @@ from . import log
 
 logger = log.get_logger()
 WARNING_MIX = ("Don't use the `kicost_variant` when using internal variants/filters")
-RegDependency.register(ToolDependency('kicost', 'KiCost', URL_KICOST, url_down=URL_KICOST+'/releases', in_debian=False,
-                                      roles=ToolDependencyRole(version=(1, 1, 7))))
 
 
 class Aggregate(Optionable):
@@ -170,8 +172,7 @@ class KiCostOptions(VariantOptions):
         # Check KiCost is available
         cmd_kicost = abspath(join(dirname(__file__), KICOST_SUBMODULE))
         if not isfile(cmd_kicost):
-            check_script(CMD_KICOST, URL_KICOST)
-            cmd_kicost = CMD_KICOST
+            cmd_kicost = self.ensure_tool('KiCost')
         # Construct the command
         cmd = [cmd_kicost, '-w', '-o', name, '-i', netlist]
         # Add the rest of input files and their variants
