@@ -1,31 +1,59 @@
 # -*- coding: utf-8 -*-
 # Copyright (c) 2022 Salvador E. Tropea
 # Copyright (c) 2022 Instituto Nacional de Tecnología Industrial
-# Copyright (c) 2022 Albin Dennevi (create_pdf_from_pages)
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
 # Base idea: https://gitlab.com/dennevi/Board2Pdf/ (Released as Public Domain)
+"""
+Dependencies:
+  - from: RSVG
+    role: Create PDF, PNG, PS and EPS formats
+    id: rsvg1
+  - from: Ghostscript
+    role: Create PNG, PS and EPS formats
+  - from: ImageMagick
+    role: Create monochrome prints and scaled PNG files
+  - from: PcbDraw
+    role: Create realistic solder masks
+  # The plot_frame_gui() needs KiAuto to print the frame
+  - from: KiAuto
+    command: pcbnew_do
+    role: Print the page frame in GUI mode
+    version: 1.6.7
+  # SVGUtils dependency. But this is also a PcbDraw dependency
+  - name: LXML
+    python_module: true
+    debian: python3-lxml
+    arch: python-lxml
+    role: mandatory
+    downloader: python
+"""
+# Direct SVG to EPS conversion is problematic.
+# If we use 72 dpi the page size is ok, but some objects (currently the solder mask) have low resolution.
+# So we create a PDF and then use GS to create the EPS files.
+#   - from: RSVG
+#     role: Create EPS format
+#     version: '2.40'
+#     id: rsvg2
 import re
 import os
 import subprocess
-from pcbnew import B_Cu, F_Cu, FromMM, IsCopperLayer, PLOT_CONTROLLER, PLOT_FORMAT_SVG, wxSize, F_Mask, B_Mask, ZONE_FILLER
-from shutil import rmtree, which
+import importlib
+from pcbnew import B_Cu, F_Cu, FromMM, IsCopperLayer, PLOT_CONTROLLER, PLOT_FORMAT_SVG, F_Mask, B_Mask, ZONE_FILLER
+import shlex
+from shutil import rmtree
 from tempfile import NamedTemporaryFile, mkdtemp
-from .svgutils.transform import fromstring, RectElement, fromfile
 from .error import KiPlotConfigurationError
 from .gs import GS
 from .optionable import Optionable
 from .out_base import VariantOptions
 from .kicad.color_theme import load_color_theme
 from .kicad.patch_svg import patch_svg_file
-from .kicad.worksheet import Worksheet, WksError
 from .kicad.config import KiConf
 from .kicad.v5_sch import SchError
 from .kicad.pcb import PCB
-from .misc import (CMD_PCBNEW_PRINT_LAYERS, URL_PCBNEW_PRINT_LAYERS, PDF_PCB_PRINT, MISSING_TOOL, W_PDMASKFAIL,
-                   KICAD5_SVG_SCALE, W_MISSTOOL, ToolDependency, ToolDependencyRole, TRY_INSTALL_CHECK)
-from .kiplot import check_script, exec_with_retry, add_extra_options
-from .registrable import RegDependency
+from .misc import PDF_PCB_PRINT, W_PDMASKFAIL, KICAD5_SVG_SCALE, W_MISSTOOL
+from .kiplot import exec_with_retry, add_extra_options
 from .create_pdf import create_pdf_from_pages
 from .macros import macros, document, output_class  # noqa: F401
 from .drill_marks import DRILL_MARKS_MAP, add_drill_marks
@@ -34,8 +62,6 @@ from . import __version__
 from . import log
 
 logger = log.get_logger()
-SVG2PDF = 'rsvg-convert'
-PDF2PS = 'pdf2ps'
 VIATYPE_THROUGH = 3
 VIATYPE_BLIND_BURIED = 2
 VIATYPE_MICROVIA = 1
@@ -43,20 +69,14 @@ POLY_FILL_STYLE = ("fill:{0}; fill-opacity:1.0; stroke:{0}; stroke-width:1; stro
                    "stroke-linejoin:round;fill-rule:evenodd;")
 DRAWING_LAYERS = ['Dwgs.User', 'Cmts.User', 'Eco1.User', 'Eco2.User']
 EXTRA_LAYERS = ['F.Fab', 'B.Fab', 'F.CrtYd', 'B.CrtYd']
-RegDependency.register(ToolDependency('pcb_print', 'RSVG tools',
-                                      'https://cran.r-project.org/web/packages/rsvg/index.html', deb='librsvg2-bin',
-                                      command=SVG2PDF,
-                                      roles=ToolDependencyRole(desc='Create PDF, PNG, EPS and PS formats')))
-RegDependency.register(ToolDependency('pcb_print', 'Ghostscript', 'https://www.ghostscript.com/',
-                                      url_down='https://github.com/ArtifexSoftware/ghostpdl-downloads/releases',
-                                      roles=ToolDependencyRole(desc='Create PS files')))
-RegDependency.register(ToolDependency('pcb_print', 'ImageMagick', 'https://imagemagick.org/', command='convert',
-                                      roles=ToolDependencyRole(desc='Create monochrome prints')))
-RegDependency.register(ToolDependency('pcb_print', 'LXML', is_python=True))
+# The following modules will be downloaded after we solve the dependencies
+# They are just helpers and we solve their dependencies
+svgutils = None  # Will be loaded during dependency check
+kicad_worksheet = None  # Also needs svgutils
 
 
 def _run_command(cmd):
-    logger.debug('- Executing: '+str(cmd))
+    logger.debug('- Executing: '+shlex.join(cmd))
     try:
         cmd_output = subprocess.check_output(cmd, stderr=subprocess.STDOUT)
     except subprocess.CalledProcessError as e:
@@ -114,39 +134,6 @@ def get_size(svg):
     return float(view_box[2]), float(view_box[3])
 
 
-def svg_to_pdf(input_folder, svg_file, pdf_file):
-    # Note: rsvg-convert uses 90 dpi but KiCad (and the docs I found) says SVG pt is 72 dpi
-    cmd = [SVG2PDF, '-d', '72', '-p', '72', '-f', 'pdf', '-o', os.path.join(input_folder, pdf_file),
-           os.path.join(input_folder, svg_file)]
-    _run_command(cmd)
-
-
-def svg_to_png(input_folder, svg_file, png_file, width):
-    cmd = [SVG2PDF, '-w', str(width), '-f', 'png', '-o', os.path.join(input_folder, png_file),
-           os.path.join(input_folder, svg_file)]
-    _run_command(cmd)
-
-
-def svg_to_eps(input_folder, svg_file, eps_file):
-    cmd = [SVG2PDF, '-d', '72', '-p', '72', '-f', 'eps', '-o', os.path.join(input_folder, eps_file),
-           os.path.join(input_folder, svg_file)]
-    _run_command(cmd)
-
-
-def pdf_to_ps(ps_file, output):
-    cmd = [PDF2PS, ps_file, output]
-    _run_command(cmd)
-
-
-def create_pdf_from_svg_pages(input_folder, input_files, output_fn):
-    svg_files = []
-    for svg_file in input_files:
-        pdf_file = svg_file.replace('.svg', '.pdf')
-        svg_to_pdf(input_folder, svg_file, pdf_file)
-        svg_files.append(os.path.join(input_folder, pdf_file))
-    create_pdf_from_pages(svg_files, output_fn)
-
-
 class LayerOptions(Layer):
     """ Data for a layer """
     def __init__(self):
@@ -203,7 +190,10 @@ class PagesOptions(Optionable):
             """ *Try to sort the layers in the same order that uses KiCad for printing """
             self.layers = LayerOptions
             """ *[list(dict)|list(string)|string] List of layers printed in this page.
-                Order is important, the last goes on top """
+                Order is important, the last goes on top.
+                You can reuse other layers lists, some options aren't used here, but they are valid """
+            self.page_id = '%02d'
+            """ Text to differentiate the pages. Use %d (like in C) to get the page number """
         self._scaling_example = 1.0
 
     def config(self, parent):
@@ -234,9 +224,14 @@ class PCB_PrintOptions(VariantOptions):
             self.output_name = None
             """ {output} """
             self.output = GS.def_global_output
-            """ *Filename for the output (%i=assembly, %x=pdf)/(%i=assembly_page_NN, %x=svg)"""
+            """ *Filename for the output (%i=assembly, %x=pdf/ps)/(%i=assembly_page_NN, %x=svg/png/eps).
+                Consult the `page_number_as_extension` and `page_id` options """
+            self.page_number_as_extension = False
+            """ When enabled the %i is always `assembly`, the %x will be NN.FORMAT (i.e. 01.png).
+                Note: page numbers can be customized using the `page_id` option for each page """
             self.hide_excluded = False
-            """ Hide components in the Fab layer that are marked as excluded by a variant """
+            """ Hide components in the Fab layer that are marked as excluded by a variant.
+                Affected by global options """
             self.color_theme = '_builtin_classic'
             """ *Selects the color theme. Only applies to KiCad 6.
                 To use the KiCad 6 default colors select `_builtin_default`.
@@ -264,7 +259,10 @@ class PCB_PrintOptions(VariantOptions):
             """ *[PDF,SVG,PNG,EPS,PS] Format for the output file/s.
                 Note that for PS you need `ghostscript` which isn't part of the default docker images """
             self.png_width = 1280
-            """ Width of the PNG in pixels """
+            """ [0,7680] Width of the PNG in pixels. Use 0 to use as many pixels as the DPI needs for the page size """
+            self.dpi = 360
+            """ [36,1200] Resolution (Dots Per Inch) for the output file. Most objects are vectors, but thing
+                like the the solder mask are handled as images by the conversion tools """
             self.colored_pads = True
             """ Plot through-hole in a different color. Like KiCad GUI does """
             self.pad_color = ''
@@ -281,6 +279,8 @@ class PCB_PrintOptions(VariantOptions):
             """ Store the temporal page and layer files in the output dir and don't delete them """
             self.force_edge_cuts = False
             """ *Add the `Edge.Cuts` to all the pages """
+            self.forced_edge_cuts_color = ''
+            """ Color used for the `force_edge_cuts` option """
             self.scaling = 1.0
             """ *Default scale factor (0 means autoscaling)"""
             self.realistic_solder_mask = True
@@ -320,7 +320,9 @@ class PCB_PrintOptions(VariantOptions):
                 self.validate_color(member)
             else:
                 setattr(self, member, getattr(self._color_theme, color))
-        if self.frame_plot_mechanism == 'plot' and GS.ki5():
+        if self.forced_edge_cuts_color:
+            self.validate_color('forced_edge_cuts_color')
+        if self.frame_plot_mechanism == 'plot' and GS.ki5:
             raise KiPlotConfigurationError("You can't use `plot` for `frame_plot_mechanism` with KiCad 5. It will crash.")
         KiConf.init(GS.pcb_file)
         if self.sheet_reference_layout:
@@ -337,30 +339,22 @@ class PCB_PrintOptions(VariantOptions):
                     if not ln.startswith('<?xml') and not ln.startswith('<!DOCTYPE svg'):
                         raise KiPlotConfigurationError("Background image must be an SVG ({})".format(self.background_image))
 
-    def filter_components(self):
-        if not self._comps:
-            return
-        comps_hash = self.get_refs_hash()
-        self.cross_modules(GS.board, comps_hash)
-        self.remove_paste_and_glue(GS.board, comps_hash)
-        if self.hide_excluded:
-            self.remove_fab(GS.board, comps_hash)
-
-    def unfilter_components(self):
-        if not self._comps:
-            return
-        comps_hash = self.get_refs_hash()
-        self.uncross_modules(GS.board, comps_hash)
-        self.restore_paste_and_glue(GS.board, comps_hash)
-        if self.hide_excluded:
-            self.restore_fab(GS.board, comps_hash)
+    def get_id_and_ext(self, n=None, id='%02d'):
+        try:
+            pn_str = id % (n+1) if n is not None else id
+        except TypeError:
+            # If id doesn't contain %d we get this exception
+            pn_str = id
+        if self.page_number_as_extension:
+            return self._expand_id, pn_str+'.'+self._expand_ext
+        return self._expand_id+'_page_'+pn_str, self._expand_ext
 
     def get_targets(self, out_dir):
         if self.format in ['SVG', 'PNG', 'EPS']:
             files = []
-            for n in range(len(self.pages)):
-                id = self._expand_id+('_page_%02d' % (n+1))
-                files.append(self.expand_filename(out_dir, self.output, id, self._expand_ext))
+            for n, p in enumerate(self.pages):
+                id, ext = self.get_id_and_ext(n, p.page_id)
+                files.append(self.expand_filename(out_dir, self.output, id, ext))
             return files
         return [self._parent.expand_filename(out_dir, self.output)]
 
@@ -432,8 +426,8 @@ class PCB_PrintOptions(VariantOptions):
         # Load the WKS
         error = None
         try:
-            ws = Worksheet.load(self.layout)
-        except (WksError, SchError) as e:
+            ws = kicad_worksheet.Worksheet.load(self.layout)
+        except (kicad_worksheet.WksError, SchError) as e:
             error = str(e)
         if error:
             raise KiPlotConfigurationError('Error reading `{}` ({})'.format(self.layout, error))
@@ -452,7 +446,7 @@ class PCB_PrintOptions(VariantOptions):
             So we print a frame using pcbnew_do export.
             We use SVG output to then generate a vectorized PDF. """
         output = os.path.join(dir_name, GS.pcb_basename+"-frame.svg")
-        check_script(CMD_PCBNEW_PRINT_LAYERS, URL_PCBNEW_PRINT_LAYERS, '1.6.7')
+        command = self.ensure_tool('KiAuto')
         # Move all the drawings away
         # KiCad 5 always prints Edge.Cuts, so we make it empty
         self.clear_layer(layer)
@@ -461,7 +455,7 @@ class PCB_PrintOptions(VariantOptions):
         # Restore the layer
         self.restore_layer()
         # Output file name
-        cmd = [CMD_PCBNEW_PRINT_LAYERS, 'export', '--output_name', output, '--monochrome', '--svg', '--pads', '0',
+        cmd = [command, 'export', '--output_name', output, '--monochrome', '--svg', '--pads', '0',
                pcb_name, dir_name, layer]
         cmd, video_remove = add_extra_options(cmd)
         # Execute it
@@ -470,7 +464,7 @@ class PCB_PrintOptions(VariantOptions):
         logger.debug('Removing temporal PCB used for frame `{}`'.format(pcb_dir))
         rmtree(pcb_dir)
         if ret:
-            logger.error(CMD_PCBNEW_PRINT_LAYERS+' returned %d', ret)
+            logger.error(command+' returned %d', ret)
             exit(PDF_PCB_PRINT)
         if video_remove:
             video_name = os.path.join(self.expand_filename_pcb(GS.out_dir), 'pcbnew_export_screencast.ogv')
@@ -488,7 +482,6 @@ class PCB_PrintOptions(VariantOptions):
         removed = []
         vias = []
         zones = GS.zones()
-        wxSize(0, 0)
         for m in GS.get_modules():
             for gi in m.GraphicalItems():
                 if gi.GetLayer() == id:
@@ -508,16 +501,27 @@ class PCB_PrintOptions(VariantOptions):
                 e.SetLayer(tmp_layer)
                 moved.append(e)
         for e in list(GS.board.Zones()):
+            found = False
+            # Remove from the set
+            layers = e.GetLayerSet()
+            if GS.layers_contains(layers, id):
+                layers.removeLayer(id)
+                e.SetLayerSet(layers)
+                removed.append(e)
+                found = True
+            # But this isn't enough if the zone is only one layer, move it to another
             if e.GetLayer() == id:
                 e.SetLayer(tmp_layer)
                 moved.append(e)
+                found = True
+            if found:
                 zones.append(e)
-        via_type = 'VIA' if GS.ki5() else 'PCB_VIA'
+        via_type = 'VIA' if GS.ki5 else 'PCB_VIA'
         for e in GS.board.GetTracks():
             if e.GetClass() == via_type:
                 vias.append((e, e.GetDrill(), e.GetWidth()))
                 e.SetDrill(0)
-                e.SetWidth(0)
+                e.SetWidth(self.min_w)
             elif e.GetLayer() == id:
                 e.SetLayer(tmp_layer)
                 moved.append(e)
@@ -540,7 +544,7 @@ class PCB_PrintOptions(VariantOptions):
         if len(zones):
             ZONE_FILLER(GS.board).Fill(zones)
         # Add it to the list
-        filelist.append((GS.pcb_basename+"-"+suffix+".svg", self.pad_color))
+        filelist.append((pc.GetPlotFileName(), self.pad_color))
 
     def plot_vias(self, la, pc, p, filelist, via_t, via_c):
         id = la._id
@@ -551,7 +555,6 @@ class PCB_PrintOptions(VariantOptions):
         removed = []
         vias = []
         zones = GS.zones()
-        wxSize(0, 0)
         for m in GS.get_modules():
             for gi in m.GraphicalItems():
                 if gi.GetLayer() == id:
@@ -568,11 +571,22 @@ class PCB_PrintOptions(VariantOptions):
                 e.SetLayer(tmp_layer)
                 moved.append(e)
         for e in list(GS.board.Zones()):
+            found = False
+            # Remove from the set
+            layers = e.GetLayerSet()
+            if GS.layers_contains(layers, id):
+                layers.removeLayer(id)
+                e.SetLayerSet(layers)
+                removed.append(e)
+                found = True
+            # But this isn't enough if the zone is only one layer, move it to another
             if e.GetLayer() == id:
                 e.SetLayer(tmp_layer)
                 moved.append(e)
+                found = True
+            if found:
                 zones.append(e)
-        via_type = 'VIA' if GS.ki5() else 'PCB_VIA'
+        via_type = 'VIA' if GS.ki5 else 'PCB_VIA'
         for e in GS.board.GetTracks():
             if e.GetClass() == via_type:
                 if e.GetViaType() == via_t:
@@ -596,7 +610,7 @@ class PCB_PrintOptions(VariantOptions):
                     w = e.GetWidth()
                     d = e.GetDrill()
                     vias.append((e, d, w, top, bottom))
-                    e.SetWidth(0)
+                    e.SetWidth(self.min_w)
             elif e.GetLayer() == id:
                 e.SetLayer(tmp_layer)
                 moved.append(e)
@@ -620,23 +634,20 @@ class PCB_PrintOptions(VariantOptions):
         if len(zones):
             ZONE_FILLER(GS.board).Fill(zones)
         # Add it to the list
-        filelist.append((GS.pcb_basename+"-"+suffix+".svg", via_c))
+        filelist.append((pc.GetPlotFileName(), via_c))
 
     def add_frame_images(self, svg, monochrome):
         if (not self.plot_sheet_reference or not self.frame_plot_mechanism == 'internal' or
            not self.last_worksheet.has_images):
             return
         if monochrome:
-            if which('convert') is None:
-                logger.error('`convert` not installed. install `imagemagick` or equivalent')
-                logger.error(TRY_INSTALL_CHECK)
-                exit(MISSING_TOOL)
+            convert_command = self.ensure_tool('ImageMagick')
             for img in self.last_worksheet.images:
                 with NamedTemporaryFile(mode='wb', suffix='.png', delete=False) as f:
                     f.write(img.data)
                     fname = f.name
                 dest = fname.replace('.png', '_gray.png')
-                _run_command(['convert', fname, '-set', 'colorspace', 'Gray', '-separate', '-average', dest])
+                _run_command([convert_command, fname, '-set', 'colorspace', 'Gray', '-separate', '-average', dest])
                 with open(dest, 'rb') as f:
                     img.data = f.read()
                 os.remove(fname)
@@ -700,13 +711,13 @@ class PCB_PrintOptions(VariantOptions):
         if not self.add_background:
             return
         if self.background_image:
-            img = fromfile(self.background_image)
+            img = svgutils.fromfile(self.background_image)
             w, h = get_size(img)
             root = img.getroot()
             root.moveto(0, 0)
             root.scale(width/w, height/h)
             svg_out.insert([root])
-        svg_out.insert(RectElement(0, 0, width, height, color=self.background_color))
+        svg_out.insert(svgutils.RectElement(0, 0, width, height, color=self.background_color))
 
     def merge_svg(self, input_folder, input_files, output_folder, output_file, p):
         """ Merge all pages into one """
@@ -714,10 +725,10 @@ class PCB_PrintOptions(VariantOptions):
         for (file, color) in input_files:
             logger.debug(' - Loading layer file '+file)
             file = os.path.join(input_folder, file)
-            new_layer = fromstring(load_svg(file, color, p.colored_holes, p.holes_color, p.monochrome))
+            new_layer = svgutils.fromstring(load_svg(file, color, p.colored_holes, p.holes_color, p.monochrome))
             width, height = get_size(new_layer)
             # Workaround for polygon fill on KiCad 5
-            if GS.ki5() and file.endswith('frame.svg'):
+            if GS.ki5 and file.endswith('frame.svg'):
                 if p.monochrome:
                     color = to_gray_hex(color)
                 self.fill_polygons(new_layer, color)
@@ -762,24 +773,21 @@ class PCB_PrintOptions(VariantOptions):
             return
         logger.debug('- Plotting realistic solder mask using PcbDraw')
         # Check PcbDraw is available
-        if which('pcbdraw') is None:
-            logger.error('`pcbdraw` not installed, needed for `realistic_solder_mask`')
-            logger.error(TRY_INSTALL_CHECK)
-            exit(MISSING_TOOL)
+        pcbdraw_command = self.ensure_tool('PcbDraw')
         # Run PcbDraw to make the heavy work (find the Edge.Cuts path and create masks)
         pcbdraw_file = os.path.join(temp_dir, out_file.replace('.svg', '-pcbdraw.svg'))
-        cmd = ['pcbdraw', '--no-warn-back', '-f', '']
+        cmd = [pcbdraw_command, '--no-warn-back', '-f', '']
         if id == B_Mask:
             cmd.append('-b')
         cmd.extend([GS.pcb_file, pcbdraw_file])
         _run_command(cmd)
         # Load the SVG created by PcbDraw
         with open(pcbdraw_file, 'rt') as f:
-            svg = fromstring(f.read())
+            svg = svgutils.fromstring(f.read())
         # Load the plot file from KiCad to get the real coordinates system
         out_file = os.path.join(temp_dir, out_file)
         with open(out_file, 'rt') as f:
-            svg_kicad = fromstring(f.read())
+            svg_kicad = svgutils.fromstring(f.read())
         view_box = svg_kicad.root.get('viewBox')
         view_box_elements = view_box.split(' ')
         # This is the paper size using the SVG precision
@@ -791,7 +799,7 @@ class PCB_PrintOptions(VariantOptions):
             # This the autocenter computation used by KiCad
             scale_x = scale_y = scale
             board_center = GS.board.GetBoundingBox().GetCenter()
-            if GS.ki5():
+            if GS.ki5:
                 # KiCad 5 uses a different precision, we must adjust
                 board_center.x = round(board_center.x*KICAD5_SVG_SCALE)
                 board_center.y = round(board_center.y*KICAD5_SVG_SCALE)
@@ -855,16 +863,83 @@ class PCB_PrintOptions(VariantOptions):
         logger.debug('- Autoscale: {}'.format(scale))
         return scale
 
+    def svg_to_pdf(self, input_folder, svg_file, pdf_file):
+        # Note: rsvg-convert uses 90 dpi but KiCad (and the docs I found) says SVG pt is 72 dpi
+        # We use a 5x scale and then reduce it to maintain the page size
+        # Note: rsvg 2.50.3 has this problem 2.54.5 doesn't, so we ensure the size is correct, not a fixed scale
+        dpi = str(self.dpi)
+        cmd = [self.rsvg_command, '-d', dpi, '-p', dpi, '-f', 'pdf', '-o', os.path.join(input_folder, pdf_file),
+               os.path.join(input_folder, svg_file)]
+        _run_command(cmd)
+
+    # We can't control the resolution in this way
+    # def svg_to_png(self, input_folder, svg_file, png_file, width):
+    #     cmd = [self.rsvg_command, '-w', str(width), '-f', 'png', '-o', os.path.join(input_folder, png_file),
+    #            os.path.join(input_folder, svg_file)]
+    #     _run_command(cmd)
+
+    # Poor resolution or wrong page size, using a PDF + GS solves it
+    # def svg_to_eps(self, input_folder, svg_file, eps_file):
+    #     cmd = [self.rsvg_command_eps, '-d', '72', '-p', '72', '-f', 'eps', '-o', os.path.join(input_folder, eps_file),
+    #            os.path.join(input_folder, svg_file)]
+    #     _run_command(cmd)
+
+    def run_gs(self, pdf_file, output, device, use_dpi=False):
+        cmd = [self.gs_command, '-q', '-dNOPAUSE', '-dBATCH', '-P-', '-dSAFER', '-sDEVICE='+device,
+               '-sOutputFile='+output, '-c', 'save', 'pop', '-f', pdf_file]
+        if use_dpi:
+            cmd.insert(1, '-r'+str(self.dpi))
+        _run_command(cmd)
+
+    def pdf_to_ps(self, pdf_file, output):
+        self.run_gs(pdf_file, output, 'ps2write')
+
+    def pdf_to_eps(self, pdf_file, output):
+        self.run_gs(pdf_file, output, 'eps2write')
+
+    def pdf_to_png(self, pdf_file, output):
+        self.run_gs(pdf_file, output, 'png16m', use_dpi=True)
+        if self.png_width:
+            # Adjust the width
+            convert_command = self.ensure_tool('ImageMagick')
+            size = str(self.png_width)+'x'
+            for n in range(len(self.pages)):
+                file = output % (n+1)
+                cmd = [convert_command, file, '-resize', size, file]
+                _run_command(cmd)
+
+    def create_pdf_from_svg_pages(self, input_folder, input_files, output_fn):
+        """ Convert individual SVG files into individual PDF files using 360 dpi.
+            Then join the individual PDF files into one PDF file scaled to the right page size. """
+        svg_files = []
+        for svg_file in input_files:
+            pdf_file = svg_file.replace('.svg', '.pdf')
+            logger.debug('- Creating {} from {}'.format(pdf_file, svg_file))
+            self.svg_to_pdf(input_folder, svg_file, pdf_file)
+            svg_files.append(os.path.join(input_folder, pdf_file))
+        logger.debug('- Joining {} into {} ({}x{})'.format(svg_files, output_fn, self.paper_w, self.paper_h))
+        create_pdf_from_pages(svg_files, output_fn, forced_width=self.paper_w)
+
+    def check_tools(self):
+        if self.format != 'SVG':
+            self.rsvg_command = self.ensure_tool('rsvg1')
+            self.gs_command = self.ensure_tool('Ghostscript')
+        # if self.format == 'EPS':
+        #    self.rsvg_command_eps = self.ensure_tool('rsvg2')
+
+    def rename_pages(self, output_dir):
+        for n, p in enumerate(self.pages):
+            id, ext = self.get_id_and_ext(n)
+            cur_name = self.expand_filename(output_dir, self.output, id, ext)
+            id, ext = self.get_id_and_ext(n, p.page_id)
+            user_name = self.expand_filename(output_dir, self.output, id, ext)
+            if cur_name != user_name and os.path.isfile(cur_name):
+                os.rename(cur_name, user_name)
+
     def generate_output(self, output):
-        if self.format != 'SVG' and which(SVG2PDF) is None:
-            logger.error('`{}` not installed. Install `librsvg2-bin` or equivalent'.format(SVG2PDF))
-            logger.error(TRY_INSTALL_CHECK)
-            exit(MISSING_TOOL)
-        if self.format == 'PS' and which(PDF2PS) is None:
-            logger.error('`{}` not installed. '.format(PDF2PS))
-            logger.error('Install `librsvg2-bin` or equivalent')
-            logger.error(TRY_INSTALL_CHECK)
-            exit(MISSING_TOOL)
+        self.check_tools()
+        # Avoid KiCad 5 complaining about fake vias diameter == drill == 0
+        self.min_w = 2 if GS.ki5 else 0
         output_dir = os.path.dirname(output)
         if self.keep_temporal_files:
             temp_dir_base = output_dir
@@ -893,11 +968,14 @@ class PCB_PrintOptions(VariantOptions):
         if self.force_edge_cuts:
             edge_layer = LayerOptions.create_layer('Edge.Cuts')
             edge_id = edge_layer._id
-            layer_id2color = self._color_theme.layer_id2color
-            if edge_id in layer_id2color:
-                edge_layer.color = layer_id2color[edge_id]
+            if self.forced_edge_cuts_color:
+                edge_layer.color = self.forced_edge_cuts_color
             else:
-                edge_layer.color = "#000000"
+                layer_id2color = self._color_theme.layer_id2color
+                if edge_id in layer_id2color:
+                    edge_layer.color = layer_id2color[edge_id]
+                else:
+                    edge_layer.color = "#000000"
         # Generate the output, page by page
         pages = []
         for n, p in enumerate(self.pages):
@@ -914,7 +992,7 @@ class PCB_PrintOptions(VariantOptions):
             p.scaling = self.set_scaling(po, p.scaling)
             po.SetNegative(p.negative_plot)
             po.SetPlotViaOnMaskLayer(not p.tent_vias)
-            if GS.ki5():
+            if GS.ki5:
                 po.SetLineWidth(FromMM(p.line_width))
                 po.SetPlotPadsOnSilkLayer(not p.exclude_pads_from_silkscreen)
             filelist = []
@@ -932,7 +1010,7 @@ class PCB_PrintOptions(VariantOptions):
                 pc.OpenPlotfile(la.suffix, PLOT_FORMAT_SVG, p.sheet)
                 pc.PlotLayer()
                 pc.ClosePlot()
-                filelist.append((GS.pcb_basename+"-"+la.suffix+".svg", la.color))
+                filelist.append((pc.GetPlotFileName(), la.color))
                 self.plot_extra_cu(id, la, pc, p, filelist)
                 self.plot_realistic_solder_mask(id, temp_dir, filelist[-1][0], filelist[-1][1], p.mirror, p.scaling)
             # 2) Plot the frame using an empty layer and 1.0 scale
@@ -949,30 +1027,38 @@ class PCB_PrintOptions(VariantOptions):
                 filelist.append((GS.pcb_basename+"-frame.svg", color))
             # 3) Stack all layers in one file
             if self.format == 'SVG':
-                id = self._expand_id+('_page_'+page_str)
-                assembly_file = self.expand_filename(output_dir, self.output, id, self._expand_ext)
+                id, ext = self.get_id_and_ext(n, p.page_id)
+                assembly_file = self.expand_filename(output_dir, self.output, id, ext)
             else:
                 assembly_file = GS.pcb_basename+".svg"
             logger.debug('- Merging layers to {}'.format(assembly_file))
             self.merge_svg(temp_dir, filelist, temp_dir, assembly_file, p)
-            if self.format in ['PNG', 'EPS']:
-                id = self._expand_id+('_page_'+page_str)
-                out_file = self.expand_filename(output_dir, self.output, id, self._expand_ext)
-                if self.format == 'PNG':
-                    svg_to_png(temp_dir, assembly_file, out_file, self.png_width)
-                else:
-                    svg_to_eps(temp_dir, assembly_file, out_file)
             pages.append(os.path.join(page_str, assembly_file))
             self.restore_title()
         # Join all pages in one file
-        if self.format in ['PDF', 'PS']:
-            logger.debug('- Creating output file {}'.format(output))
+        if self.format != 'SVG':
             if self.format == 'PDF':
-                create_pdf_from_svg_pages(temp_dir_base, pages, output)
+                logger.debug('- Creating output file {}'.format(output))
+                self.create_pdf_from_svg_pages(temp_dir_base, pages, output)
             else:
-                ps_file = os.path.join(temp_dir, GS.pcb_basename+'.ps')
-                create_pdf_from_svg_pages(temp_dir_base, pages, ps_file)
-                pdf_to_ps(ps_file, output)
+                logger.debug('- Creating output files')
+                # PS and EPS using Ghostscript
+                # Create a PDF (but in a temporal place)
+                pdf_file = os.path.join(temp_dir, GS.pcb_basename+'_joined.pdf')
+                self.create_pdf_from_svg_pages(temp_dir_base, pages, pdf_file)
+                if self.format == 'PS':
+                    # Use GS to create one PS
+                    self.pdf_to_ps(pdf_file, output)
+                else:  # EPS and PNG
+                    id, ext = self.get_id_and_ext()
+                    out_file = self.expand_filename(output_dir, self.output, id, ext, make_safe=False)
+                    if self.format == 'EPS':
+                        # Use GS to create one EPS per page
+                        self.pdf_to_eps(pdf_file, out_file)
+                    else:
+                        # Use GS to create one PNG per page and then scale to the wanted width
+                        self.pdf_to_png(pdf_file, out_file)
+                    self.rename_pages(output_dir)
         # Remove the temporal files
         if not self.keep_temporal_files:
             rmtree(temp_dir_base)
@@ -980,9 +1066,14 @@ class PCB_PrintOptions(VariantOptions):
 
     def run(self, output):
         super().run(output)
-        self.filter_components()
+        self.ensure_tool('LXML')
+        global svgutils
+        svgutils = importlib.import_module('.svgutils.transform', package=__package__)
+        global kicad_worksheet
+        kicad_worksheet = importlib.import_module('.kicad.worksheet', package=__package__)
+        self.filter_pcb_components(GS.board)
         self.generate_output(output)
-        self.unfilter_components()
+        self.unfilter_pcb_components(GS.board)
 
 
 @output_class
@@ -1002,22 +1093,24 @@ class PCB_Print(BaseOutput):  # noqa: F821
     @staticmethod
     def get_conf_examples(name, layers, templates):
         outs = []
-        if len(DRAWING_LAYERS) < 10 and GS.ki6():
+        if len(DRAWING_LAYERS) < 10 and GS.ki6:
             DRAWING_LAYERS.extend(['User.'+str(c+1) for c in range(9)])
         extra = {la._id for la in Layer.solve(EXTRA_LAYERS)}
         disabled = set()
         # Check we can use PcbDraw
-        realistic_solder_mask = which('pcbdraw') is not None
+        realistic_solder_mask = GS.check_tool(name, 'PcbDraw') is not None
         if not realistic_solder_mask:
             logger.warning(W_MISSTOOL+'Missing PcbDraw tool, disabling `realistic_solder_mask`')
         # Check we can convert SVGs
-        if which(SVG2PDF) is None:
-            logger.warning(W_MISSTOOL+'Missing {} tool, disabling most printed formats'.format(SVG2PDF))
+        if GS.check_tool(name, 'rsvg1') is None:
+            logger.warning(W_MISSTOOL+'Disabling most printed formats')
             disabled |= {'PDF', 'PNG', 'EPS', 'PS'}
         # Check we can convert to PS
-        if which(PDF2PS) is None:
-            logger.warning(W_MISSTOOL+'Missing {} tool, disabling postscript printed format'.format(PDF2PS))
-            disabled.add('PS')
+        if GS.check_tool(name, 'Ghostscript') is None:
+            logger.warning(W_MISSTOOL+'Disabling postscript/PDF printed format')
+            disabled |= {'PDF', 'PNG', 'EPS', 'PS'}
+        if GS.check_tool(name, 'ImageMagick') is None:
+            disabled |= {'PNG'}
         # Generate one output for each format
         for fmt in ['PDF', 'SVG', 'PNG', 'EPS', 'PS']:
             if fmt in disabled:
