@@ -31,6 +31,12 @@ def _ref_key(ref_str):
     return [pre, 0 if suf == '?' else int(suf)]
 
 
+def check_names(top, bot):
+    if top == bot:
+        raise KiPlotConfigurationError("Asking for two separated files, but both with the same name.\n"
+                                       "Try using %i in the name.")
+
+
 class PosColumns(Optionable):
     """ Which columns we want and its names """
     def __init__(self):
@@ -60,11 +66,15 @@ class PositionOptions(VariantOptions):
             self.only_smd = True
             """ *Only include the surface mount components """
             self.output = GS.def_global_output
-            """ *Output file name (%i='top_pos'|'bottom_pos'|'both_pos', %x='pos'|'csv') """
+            """ *Output file name (%i='top_pos'|'bottom_pos'|'both_pos', %x='pos'|'csv').
+                Important: when using separate files you must use `%i` to differentiate them """
             self.units = 'millimeters'
             """ *[millimeters,inches,mils] Units used for the positions. Affected by global options """
             self.columns = PosColumns
             """ [list(dict)|list(string)] Which columns are included in the output """
+            self.right_digits = 4
+            """ number of digits for mantissa part of coordinates (0 is auto) """
+            self.columns = PosColumns
             self.bottom_negative_x = False
             """ Use negative X coordinates for footprints on bottom layer """
             self.use_aux_axis_as_origin = True
@@ -93,15 +103,19 @@ class PositionOptions(VariantOptions):
             self.columns = new_columns
         self._expand_ext = 'pos' if self.format == 'ASCII' else 'csv'
 
-    def _do_position_plot_ascii(self, output_dir, columns, modulesStr, maxSizes):
+    def _do_position_plot_ascii(self, output_dir, columns, modulesStr, maxSizes, modules_side):
         topf = None
         botf = None
         bothf = None
         if self.separate_files_for_front_and_back:
-            topf = open(self.expand_filename(output_dir, self.output, 'top_pos', 'pos'), 'w')
-            botf = open(self.expand_filename(output_dir, self.output, 'bottom_pos', 'pos'), 'w')
+            topf_name = self.expand_filename(output_dir, self.output, 'top_pos', 'pos')
+            botf_name = self.expand_filename(output_dir, self.output, 'bottom_pos', 'pos')
+            check_names(topf_name, botf_name)
+            topf = open(topf_name, 'w')
+            botf = open(botf_name, 'w')
         else:
-            bothf = open(self.expand_filename(output_dir, self.output, 'both_pos', 'pos'), 'w')
+            fname = self.expand_filename(output_dir, self.output, 'both_pos', 'pos')
+            bothf = open(fname, 'w')
 
         files = [f for f in [topf, botf, bothf] if f is not None]
         for f in files:
@@ -128,13 +142,8 @@ class PositionOptions(VariantOptions):
         # Account for the "# " at the start of the comment column
         maxSizes[0] = maxSizes[0] + 2
 
-        for m in modulesStr:
-            file = bothf
-            if file is None:
-                if m[-1] == "top":
-                    file = topf
-                else:
-                    file = botf
+        for (m, is_bottom) in zip(modulesStr, modules_side):
+            file = bothf if bothf is not None else (botf if is_bottom else topf)
             for idx, col in enumerate(m):
                 if idx > 0:
                     file.write("   ")
@@ -151,13 +160,16 @@ class PositionOptions(VariantOptions):
         if bothf is not None:
             bothf.close()
 
-    def _do_position_plot_csv(self, output_dir, columns, modulesStr):
+    def _do_position_plot_csv(self, output_dir, columns, modulesStr, modules_side):
         topf = None
         botf = None
         bothf = None
         if self.separate_files_for_front_and_back:
-            topf = open(self.expand_filename(output_dir, self.output, 'top_pos', 'csv'), 'w')
-            botf = open(self.expand_filename(output_dir, self.output, 'bottom_pos', 'csv'), 'w')
+            topf_name = self.expand_filename(output_dir, self.output, 'top_pos', 'csv')
+            botf_name = self.expand_filename(output_dir, self.output, 'bottom_pos', 'csv')
+            check_names(topf_name, botf_name)
+            topf = open(topf_name, 'w')
+            botf = open(botf_name, 'w')
         else:
             bothf = open(self.expand_filename(output_dir, self.output, 'both_pos', 'csv'), 'w')
 
@@ -167,13 +179,8 @@ class PositionOptions(VariantOptions):
             f.write(",".join(columns))
             f.write("\n")
 
-        for m in modulesStr:
-            file = bothf
-            if file is None:
-                if m[-1] == "top":
-                    file = topf
-                else:
-                    file = botf
+        for (m, is_bottom) in zip(modulesStr, modules_side):
+            file = bothf if bothf is not None else (botf if is_bottom else topf)
             file.write(",".join('{}'.format(e) for e in m))
             file.write("\n")
 
@@ -202,7 +209,7 @@ class PositionOptions(VariantOptions):
 
     @staticmethod
     def get_attr_tests():
-        if GS.ki5():
+        if GS.ki5:
             return PositionOptions.is_pure_smd_5, PositionOptions.is_not_virtual_5
         return PositionOptions.is_pure_smd_6, PositionOptions.is_not_virtual_6
 
@@ -221,6 +228,7 @@ class PositionOptions(VariantOptions):
         # Format all strings
         comps_hash = self.get_refs_hash()
         modules = []
+        modules_side = []
         is_pure_smd, is_not_virtual = self.get_attr_tests()
         quote_char = '"' if self.format == 'CSV' else ''
         x_origin = 0.0
@@ -243,8 +251,10 @@ class PositionOptions(VariantOptions):
                     footprint = c.footprint
                     is_bottom = c.bottom
                     rotation = c.footprint_rot
-                    center_x = c.footprint_x
-                    center_y = c.footprint_y
+                    # Here we can't use c.footprint_x/y because this doesn't work for panels
+                    center = GS.get_center(m)
+                    center_x = center.x
+                    center_y = center.y
             if value is None:
                 value = m.GetValue()
                 footprint = str(m.GetFPID().GetLibItemName())  # pcbnew.UTF8 type
@@ -257,6 +267,10 @@ class PositionOptions(VariantOptions):
             if (self.only_smd and is_pure_smd(m)) or (not self.only_smd and (is_not_virtual(m) or self.include_virtual)):
                 # KiCad: PLACE_FILE_EXPORTER::GenPositionData() in export_footprints_placefile.cpp
                 row = []
+                if self.right_digits != 0:
+                    float_format = "{{:.{}f}}".format(self.right_digits)
+                else:
+                    float_format = "{}"
                 for k in self.columns:
                     if k == 'Ref':
                         row.append(quote_char+ref+quote_char)
@@ -268,14 +282,15 @@ class PositionOptions(VariantOptions):
                         pos_x = (center_x - x_origin) * conv
                         if self.bottom_negative_x and is_bottom:
                             pos_x = -pos_x
-                        row.append("{:.4f}".format(pos_x))
+                        row.append(float_format.format(pos_x, rd=self.right_digits))
                     elif k == 'PosY':
-                        row.append("{:.4f}".format(-(center_y - y_origin) * conv))
+                        row.append(float_format.format(-(center_y - y_origin) * conv, rd=self.right_digits))
                     elif k == 'Rot':
-                        row.append("{:.4f}".format(rotation))
+                        row.append(float_format.format(rotation, rd=self.right_digits))
                     elif k == 'Side':
                         row.append("bottom" if is_bottom else "top")
                 modules.append(row)
+                modules_side.append(is_bottom)
         # Find max width for all columns
         maxlengths = []
         for col, name in enumerate(columns):
@@ -285,9 +300,9 @@ class PositionOptions(VariantOptions):
             maxlengths.append(max_l)
         # Note: the parser already checked the format is ASCII or CSV
         if self.format == 'ASCII':
-            self._do_position_plot_ascii(output_dir, columns, modules, maxlengths)
+            self._do_position_plot_ascii(output_dir, columns, modules, maxlengths, modules_side)
         else:  # if self.format == 'CSV':
-            self._do_position_plot_csv(output_dir, columns, modules)
+            self._do_position_plot_csv(output_dir, columns, modules, modules_side)
 
 
 @output_class
