@@ -3,8 +3,15 @@
 # Copyright (c) 2022 Instituto Nacional de Tecnología Industrial
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
+"""
+Dependencies:
+  - name: QRCodeGen
+    role: mandatory
+    python_module: true
+    downloader: python
+    debian: python3-qrcodegen
+"""
 import os
-from qrcodegen import QrCode
 from tempfile import NamedTemporaryFile
 from .gs import GS
 from .optionable import BaseOptions, Optionable
@@ -12,19 +19,16 @@ from .error import KiPlotConfigurationError
 from .kicad.sexpdata import Symbol, dumps, Sep, load, SExpData, sexp_iter
 from .kicad.v6_sch import DrawRectangleV6, PointXY, Stroke, Fill, SchematicFieldV6, FontEffects
 from .kiplot import load_board
-from .misc import ToolDependency, ToolDependencyRole
-from .registrable import RegDependency
 from .macros import macros, document, output_class  # noqa: F401
 from . import log
+try:
+    import qrcodegen
+except ImportError:
+    qrcodegen = None
 
-QR_ECCS = {'low': QrCode.Ecc.LOW,
-           'medium': QrCode.Ecc.MEDIUM,
-           'quartile': QrCode.Ecc.QUARTILE,
-           'high': QrCode.Ecc.HIGH}
 logger = log.get_logger()
 TO_SEPARATE = {'kicad_pcb', 'general', 'title_block', 'layers', 'setup', 'pcbplotparams', 'net_class', 'module',
                'kicad_sch', 'lib_symbols', 'symbol', 'sheet', 'sheet_instances', 'symbol_instances'}
-RegDependency.register(ToolDependency('qr_lib', 'QRCodeGen', is_python=True, roles=ToolDependencyRole()))
 
 
 def is_symbol(name, sexp):
@@ -90,7 +94,6 @@ class QRCodeOptions(Optionable):
 
     def config(self, parent):
         super().config(parent)
-        self.correction_level = QR_ECCS[self.correction_level]
         self.layer = 'F.SilkS' if self.layer == 'silk' else 'F.Cu'
 
 
@@ -315,14 +318,29 @@ class QR_LibOptions(BaseOptions):
             f.write(dumps(lib))
             f.write('\n')
 
+    @staticmethod
+    def find_layer(sexp, def_layer):
+        """ Determine the layer.
+            The user could move the footprint to the bottom layer """
+        # Look for the polys
+        poly = next(filter(lambda s: is_symbol('fp_poly', s), sexp), None)
+        if not poly:
+            return def_layer
+        layer = next(filter(lambda s: is_symbol('layer', s), poly), None)
+        if not layer or len(layer) != 2:
+            return def_layer
+        return layer[1]
+
     def update_footprint(self, name, sexp, qr):
         logger.debug('- Updating QR footprint: '+name)
         # Compute the size
         qrc, size, full_size, center, size_rect = compute_size(qr, is_sch=False)
+        # In which layer are the drawings? (default to the original)
+        layer = self.find_layer(sexp, qr.layer)
         # Remove old drawing
         sexp[:] = list(filter(lambda s: not is_symbol('fp_poly', s), sexp))
         # Add the new drawings
-        sexp.extend(self.qr_draw_fp(size, size_rect, center, qrc, qr.pcb_negative, qr.layer, do_sep=False))
+        sexp.extend(self.qr_draw_fp(size, size_rect, center, qrc, qr.pcb_negative, layer, do_sep=False))
         # Update the fields
         for s in sexp:
             if (is_symbol('fp_text', s) and len(s) > 2 and isinstance(s[1], Symbol) and s[1].value() == 'user' and
@@ -377,7 +395,7 @@ class QR_LibOptions(BaseOptions):
             os.remove(tmp_pcb)
             GS.make_bkp(GS.pcb_file)
             prl = None
-            if GS.ki6():
+            if GS.ki6:
                 # KiCad 6 is destroying the PRL ...
                 prl_name = GS.pcb_no_ext+'.kicad_prl'
                 if os.path.isfile(prl_name):
@@ -474,6 +492,14 @@ class QR_LibOptions(BaseOptions):
         return sheets
 
     def run(self, output):
+        global qrcodegen
+        if qrcodegen is None:
+            qrcodegen = self.ensure_tool('QRCodeGen')
+        # Now we are sure we have qrcodegen
+        QR_ECCS = {'low': qrcodegen.QrCode.Ecc.LOW,
+                   'medium': qrcodegen.QrCode.Ecc.MEDIUM,
+                   'quartile': qrcodegen.QrCode.Ecc.QUARTILE,
+                   'high': qrcodegen.QrCode.Ecc.HIGH}
         if self.use_sch_dir:
             self._odir_sch = GS.sch_dir
             self._odir_pcb = GS.pcb_dir
@@ -482,11 +508,11 @@ class QR_LibOptions(BaseOptions):
         # Create the QR codes
         for qr in self.qrs:
             qr._text_sch = self.expand_filename_both(qr.text, make_safe=False)
-            qr._code_sch = QrCode.encode_text(qr._text_sch, qr.correction_level)
+            qr._code_sch = qrcodegen.QrCode.encode_text(qr._text_sch, QR_ECCS[qr.correction_level])
             qr._text_pcb = self.expand_filename_both(qr.text, is_sch=False, make_safe=False)
-            qr._code_pcb = QrCode.encode_text(qr._text_pcb, qr.correction_level)
+            qr._code_pcb = qrcodegen.QrCode.encode_text(qr._text_pcb, QR_ECCS[qr.correction_level])
         # Create the symbols
-        if GS.ki5():
+        if GS.ki5:
             self.symbol_lib_k5()
         else:
             self.symbol_lib_k6()
@@ -509,7 +535,7 @@ class QR_LibOptions(BaseOptions):
             # PCB
             self.update_footprints(known_qrs)
             # Schematic
-            if GS.ki6():
+            if GS.ki6:
                 # KiCad 5 reads the lib, but KiCad 6 is more like the PCB
                 assert GS.sch_file is not None
                 sheets = self.load_k6_sheets(GS.sch_file)
