@@ -33,6 +33,11 @@ class BasePreFlight(Registrable):
     def add_preflight(o_pre):
         BasePreFlight._in_use[o_pre._name] = o_pre
 
+    @staticmethod
+    def add_preflights(pre):
+        for p in pre:
+            BasePreFlight._in_use[p._name] = p
+
     @staticmethod
     def get_preflight(name):
         return BasePreFlight._in_use.get(name)
@@ -133,7 +138,21 @@ class BasePreFlight(Registrable):
         return Optionable.expand_filename_both(self, out_dir, is_sch=self._sch_related)
 
     def _find_variant(self):
-        return ''
+        # Preflights doesn't have a variant, but we could have one global default
+        if hasattr(self, '_variant') and self._variant:
+            return self._variant.file_id
+        return Optionable._find_global_variant()
 
     def _find_variant_name(self):
-        return ''
+        # Preflights doesn't have a variant, but we could have one global default
+        if hasattr(self, '_variant') and self._variant:
+            return self._variant.name
+        return Optionable._find_global_variant_name()
+
+    def ensure_tool(self, name):
+        """ Looks for a mandatory dependency """
+        return GS.check_tool_dep(self._name, name, fatal=True)
+
+    def check_tool(self, name):
+        """ Looks for a dependency """
+        return GS.check_tool_dep(self._name, name, fatal=False)
