@@ -3,15 +3,17 @@
 # Copyright (c) 2021-2022 Instituto Nacional de Tecnología Industrial
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
+"""
+Dependencies:
+  - from: Git
+    role: Find commit hash and/or date
+"""
 from .gs import GS
 from .pre_any_replace import TagReplaceBase, Base_ReplaceOptions, Base_Replace
-from .registrable import RegDependency
-from .misc import git_dependency
 from .macros import macros, document, pre_class  # noqa: F401
 from . import log
 
 logger = log.get_logger()
-RegDependency.register(git_dependency('pcb_replace'))
 
 
 class TagReplacePCB(TagReplaceBase):
