@@ -1,19 +1,21 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2021 Salvador E. Tropea
-# Copyright (c) 2021 Instituto Nacional de Tecnología Industrial
+# Copyright (c) 2021-2022 Salvador E. Tropea
+# Copyright (c) 2021-2022 Instituto Nacional de Tecnología Industrial
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
+"""
+Dependencies:
+  - from: Git
+    role: Find commit hash and/or date
+"""
 import os
 from .gs import GS
 from .kiplot import load_sch
 from .pre_any_replace import TagReplaceBase, Base_ReplaceOptions, Base_Replace
-from .registrable import RegDependency
-from .misc import git_dependency
 from .macros import macros, document, pre_class  # noqa: F401
 from . import log
 
 logger = log.get_logger()
-RegDependency.register(git_dependency('sch_replace'))
 
 
 class TagReplaceSCH(TagReplaceBase):
@@ -54,7 +56,7 @@ class SCH_Replace(Base_Replace):  # noqa: F821
         if o.date_command:
             # Convert it into another replacement
             t = TagReplaceSCH()
-            if GS.ki5():
+            if GS.ki5:
                 t.tag = r'^Date ("(?:[^"]|\\")*")$'
                 t.before = 'Date "'
                 t.after = '"'
