@@ -3,21 +3,27 @@
 # Copyright (c) 2022 Instituto Nacional de Tecnología Industrial
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
-import os
-import sys
+"""
+Dependencies:
+  - from: Git
+    role: Find commit hash and/or date
+"""
 import json
+import os
+import pcbnew
+import re
 from subprocess import run, PIPE
+import sys
 from .error import KiPlotConfigurationError
-from .misc import FAILED_EXECUTE, W_EMPTREP, git_dependency
+from .misc import FAILED_EXECUTE, W_EMPTREP
 from .optionable import Optionable
 from .pre_base import BasePreFlight
 from .gs import GS
-from .registrable import RegDependency
 from .macros import macros, document, pre_class  # noqa: F401
 from . import log
 
 logger = log.get_logger()
-RegDependency.register(git_dependency('set_text_variables'))
+re_git = re.compile(r'([^a-zA-Z_]|^)(git) ')
 
 
 class KiCadVariable(Optionable):
@@ -91,7 +97,7 @@ class Set_Text_Variables(BasePreFlight):  # noqa: F821
         o = self._value
         if len(o) == 0:
             return
-        if GS.ki5():
+        if GS.ki5:
             raise KiPlotConfigurationError("The `set_text_variables` preflight is for KiCad 6 or newer")
         pro_name = GS.pro_file
         if not pro_name or not os.path.isfile(pro_name):
@@ -111,15 +117,18 @@ class Set_Text_Variables(BasePreFlight):  # noqa: F821
             os.environ['KIBOT_SCH_NAME'] = GS.sch_file
         for r in o:
             text = r.text
-            if not text:
-                cmd = ['/bin/bash', '-c', r.command]
+            if not text and r.command:
+                command = r.command
+                if re_git.search(command):
+                    git_command = self.ensure_tool('git')
+                    command = re_git.sub(r'\1'+git_command+' ', command)
+                cmd = ['/bin/bash', '-c', command]
                 result = run(cmd, stdout=PIPE, stderr=PIPE, universal_newlines=True)
                 if result.returncode:
                     logger.error('Failed to execute:\n{}\nreturn code {}'.format(r.command, result.returncode))
                     sys.exit(FAILED_EXECUTE)
                 if not result.stdout:
-                    logger.warning(W_EMPTREP+"Empty value from `{}` skipping it".format(r.command))
-                    continue
+                    logger.warning(W_EMPTREP+"Empty value from `{}`".format(r.command))
                 text = result.stdout.strip()
             text = r.before + text + r.after
             logger.debug('  - ' + r.name + ' -> ' + text)
@@ -139,5 +148,10 @@ class Set_Text_Variables(BasePreFlight):  # noqa: F821
         GS.make_bkp(pro_name)
         with open(pro_name, 'wt') as f:
             f.write(json.dumps(data, sort_keys=True, indent=2))
-        # Force the PCB reload (will reload the project file)
-        GS.board = None
+        if GS.board:
+            # Force a project reload
+            sm = pcbnew.GetSettingsManager()
+            sm.UnloadProject(GS.board.GetProject(), False)
+            assert sm.LoadProject(pro_name)
+            # Force the PCB reload (will reload the project file)
+            GS.board = None
